@@ -76,8 +76,12 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
     ----------
     _input_sizes : list
         Stores the size of the inputs at each level.
+    _static_input_sizes : list
+        Stores the size of the inputs at each level for inputs added outside of setup.
     _nfi : float
         number of levels of fidelity
+    _training_input : dict
+        Training data for inputs.
     """
 
     def __init__(self, **kwargs):
@@ -97,6 +101,8 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
         self._training_input = nfi * [np.empty(0)]
         self._input_sizes = nfi * [0]
 
+        self._static_input_sizes = nfi * [0]
+
     def initialize(self):
         """
         Declare options.
@@ -106,8 +112,13 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
         self.options.declare('nfi', types=int, default=1, lower=1,
                              desc='Number of levels of fidelity.')
 
+    def _setup_procs(self, pathname, comm, mode):
+        self._input_sizes = list(self._static_input_sizes)
+
+        super(MultiFiMetaModelUnStructuredComp, self)._setup_procs(pathname, comm, mode)
+
     def add_input(self, name, val=1.0, shape=None, src_indices=None, flat_src_indices=None,
-                  units=None, desc='', var_set=0):
+                  units=None, desc=''):
         """
         Add an input variable to the component.
 
@@ -135,20 +146,20 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
             during execution. Default is None, which means it is unitless.
         desc : str
             description of the variable
-        var_set : hashable object
-            For advanced users only. ID or color for this variable, relevant for
-            reconfigurability. Default is 0.
         """
         item = MultiFiMetaModelUnStructuredComp
         metadata = super(item, self).add_input(name, val, shape=shape, src_indices=src_indices,
                                                flat_src_indices=flat_src_indices, units=units,
-                                               desc=desc, var_set=var_set)
+                                               desc=desc)
         if self.options['vec_size'] > 1:
             input_size = metadata['value'][0].size
         else:
             input_size = metadata['value'].size
 
-        self._input_sizes[0] = self._input_size
+        if self._static_mode:
+            self._static_input_sizes[0] += input_size
+        else:
+            self._input_sizes[0] += input_size
 
         # Add train:<invar>_fi<n>
         for fi in range(self._nfi):
@@ -156,10 +167,13 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
                 name_with_fi = 'train:' + _get_name_fi(name, fi)
                 self.options.declare(
                     name_with_fi, default=None, desc='Training data for %s' % name_with_fi)
-                self._input_sizes[fi] += input_size
+                if self._static_mode:
+                    self._static_input_sizes[fi] += input_size
+                else:
+                    self._input_sizes[fi] += input_size
 
     def add_output(self, name, val=1.0, surrogate=None, shape=None, units=None, res_units=None,
-                   desc='', lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=1.0, var_set=0):
+                   desc='', lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=1.0):
         """
         Add an output variable to the component.
 
@@ -201,16 +215,12 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
         res_ref : float
             Scaling parameter. The value in the user-defined res_units of this output's residual
             when the scaled value is 1. Default is 1.
-        var_set : hashable object
-            For advanced users only. ID or color for this variable, relevant for reconfigurability.
-            Default is 0.
         """
         super(MultiFiMetaModelUnStructuredComp, self).add_output(name, val, shape=shape,
                                                                  units=units, res_units=res_units,
                                                                  desc=desc, lower=lower,
                                                                  upper=upper, ref=ref,
                                                                  ref0=ref0, res_ref=res_ref,
-                                                                 var_set=var_set,
                                                                  surrogate=surrogate)
         self._training_output[name] = self._nfi * [np.empty(0)]
 
