@@ -9,201 +9,252 @@ import numpy as np
 from numpy.testing import assert_almost_equal
 import scipy
 
-from parameterized import parameterized
+try:
+    from parameterized import parameterized
+except ImportError:
+    from openmdao.utils.assert_utils import SkipParameterized as parameterized
 
 from openmdao.api import IndepVarComp, Group, Problem, ExecComp
 from openmdao.components.exec_comp import _expr_dict
 from openmdao.utils.assert_utils import assert_rel_error
 
-_ufunc_test_data = {'abs': {'str': 'f=abs(x)',
-                            'check_func': np.abs,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)}}},
-                    'acos': {'str': 'f=acos(x)',
-                             'check_func': np.arccos,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)-0.5}}},
-                    'arccos': {'str': 'f=arccos(x)',
-                               'check_func': np.arccos,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)-0.5}}},
-                    'arccosh': {'str': 'f=arccosh(x)',
-                                'check_func': np.arccosh,
-                                'args': {'f': {'value': np.zeros(6)},
-                                         'x': {'value': 1.1+np.random.random(6)}}},
-                    'acosh': {'str': 'f=acosh(x)',
-                              'check_func': np.arccosh,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': 1.1+np.random.random(6)}}},
-                    'arange': {'str': 'f=arange(0,10,2)',
-                               'check_val': np.arange(0, 10, 2),
-                               'args': {'f': {'value': np.zeros(5)}}},
-                    'arcsin': {'str': 'f=arcsin(x)',
-                               'check_func': np.arcsin,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)-.5}}},
-                    'arcsinh': {'str': 'f=arcsinh(x)',
-                                'check_func': np.arcsinh,
-                                'args': {'f': {'value': np.zeros(6)},
-                                         'x': {'value': np.random.random(6)}}},
-                    'asinh': {'str': 'f=asinh(x)',
-                              'check_func': np.arcsinh,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': np.random.random(6)}}},
-                    'asin': {'str': 'f=asin(x)',
-                             'check_func': np.arcsin,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)-.5}}},
-                    'arctan': {'str': 'f=arctan(x)',
-                               'check_func': np.arctan,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
-                    'atan': {'str': 'f=atan(x)',
-                             'check_func': np.arctan,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)}}},
-                    'cos': {'str': 'f=cos(x)',
-                            'check_func': np.cos,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)}}},
-                    'cosh': {'str': 'f=cosh(x)',
-                             'check_func': np.cosh,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)}}},
-                    'dot': {'str': 'f=dot(x, y)',
-                            'check_func': np.dot,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)},
-                                     'y': {'value': np.random.random(6)}}},
-                    'e': {'str': 'f=e',
-                          'check_val': np.e,
-                          'args': {'f': {'value': 0.0}}},
-                    'erf': {'str': 'f=erf(x)',
-                            'check_func': scipy.special.erf,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)}}},
-                    'erfc': {'str': 'f=erfc(x)',
-                             'check_func': scipy.special.erfc,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)}}},
-                    'exp': {'str': 'f=exp(x)',
-                            'check_func': np.exp,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)}}},
-                    'expm1': {'str': 'f=expm1(x)',
-                              'check_func': np.expm1,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': np.random.random(6)}}},
-                    'factorial': {'str': 'f=factorial(x)',
-                                  'check_func': scipy.special.factorial,
-                                  'args': {'f': {'value': np.zeros(6)},
-                                           'x': {'value': np.random.random(6)}}},
-                    'fmax': {'str': 'f=fmax(x, y)',
-                             'check_func': np.fmax,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)},
-                                      'y': {'value': np.random.random(6)}}},
-                    'fmin': {'str': 'f=fmin(x, y)',
-                             'check_func': np.fmin,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)},
-                                      'y': {'value': np.random.random(6)}}},
-                    'inner': {'str': 'f=inner(x, y)',
-                              'check_func': np.inner,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': np.random.random(6)},
-                                       'y': {'value': np.random.random(6)}}},
-                    'isinf': {'str': 'f=isinf(x)',
-                              'check_func': np.isinf,
-                              'args': {'f': {'value': np.zeros(3)},
-                                       'x': {'value': [0, np.inf, 5.0]}}},
-                    'isnan': {'str': 'f=isnan(x)',
-                              'check_func': np.isnan,
-                              'args': {'f': {'value': np.zeros(3)},
-                                       'x': {'value': [0, np.nan, np.nan]}}},
-                    'kron': {'str': 'f=kron(x, y)',
-                             'check_func': np.kron,
-                             'args': {'f': {'value': np.zeros(36)},
-                                      'x': {'value': np.random.random(6)},
-                                      'y': {'value': np.random.random(6)}}},
-                    'linspace': {'str': 'f=linspace(0,10,50)',
-                                 'check_val': np.linspace(0, 10, 50),
-                                 'args': {'f': {'value': np.zeros(50)}}},
-                    'log': {'str': 'f=log(x)',
-                            'check_func': np.log,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)+0.1}}},
-                    'log10': {'str': 'f=log10(x)',
-                              'check_func': np.log10,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': np.random.random(6)+0.1}}},
-                    'log1p': {'str': 'f=log1p(x)',
-                              'check_func': np.log1p,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': np.random.random(6)}}},
-                    'matmul': {'str': 'f=matmul(x, y)',
-                               'check_func': np.matmul,
-                               'args': {'f': {'value': np.zeros((3, 1))},
-                                        'x': {'value': np.random.random((3, 3))},
-                                        'y': {'value': np.random.random((3, 1))}}},
-                    'maximum': {'str': 'f=maximum(x, y)',
-                                'check_func': np.maximum,
-                                'args': {'f': {'value': np.zeros(6)},
-                                         'x': {'value': np.random.random(6)},
-                                         'y': {'value': np.random.random(6)}}},
-                    'minimum': {'str': 'f=minimum(x, y)',
-                                'check_func': np.minimum,
-                                'args': {'f': {'value': np.zeros(6)},
-                                         'x': {'value': np.random.random(6)},
-                                         'y': {'value': np.random.random(6)}}},
-                    'ones': {'str': 'f=ones(21)',
-                             'check_val': np.ones(21),
-                             'args': {'f': {'value': np.zeros(21)}}},
-                    'outer': {'str': 'f=outer(x, y)',
-                              'check_func': np.outer,
-                              'args': {'f': {'value': np.zeros((6, 6))},
-                                       'x': {'value': np.random.random(6)},
-                                       'y': {'value': np.random.random(6)}}},
-                    'pi': {'str': 'f=pi',
-                           'check_val': np.pi,
-                           'args': {'f': {'value': 0.0}}},
-                    'power': {'str': 'f=power(x, y)',
-                              'check_func': np.power,
-                              'args': {'f': {'value': np.zeros(6)},
-                                       'x': {'value': np.random.random(6)},
-                                       'y': {'value': np.random.random(6)+1.0}}},
-                    'prod': {'str': 'f=prod(x)',
-                             'check_func': np.prod,
-                             'args': {'f': {'value': 0.0},
-                                      'x': {'value': np.random.random(6)}}},
-                    'sin': {'str': 'f=sin(x)',
-                            'check_func': np.sin,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)}}},
-                    'sinh': {'str': 'f=sinh(x)',
-                             'check_func': np.sinh,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)}}},
-                    'sum': {'str': 'f=sum(x)',
-                            'check_func': np.sum,
-                            'args': {'f': {'value': 0.0},
-                                     'x': {'value': np.random.random(6)}}},
-                    'tan': {'str': 'f=tan(x)',
-                            'check_func': np.tan,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)}}},
-                    'tanh': {'str': 'f=tanh(x)',
-                             'check_func': np.tanh,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)}}},
-                    'tensordot': {'str': 'f=tensordot(x, y)',
-                                  'check_func': np.tensordot,
-                                  'args': {'f': {'value': 0.0},
-                                           'x': {'value': np.random.random((6, 6))},
-                                           'y': {'value': np.random.random((6, 6))}}},
-                    'zeros': {'str': 'f=zeros(21)',
-                              'check_val': np.zeros(21),
-                              'args': {'f': {'value': np.zeros(21)}}}, }
+_ufunc_test_data = {
+    'abs': {
+        'str': 'f=abs(x)',
+        'check_func': np.abs,
+        'args': { 'f': {'value': np.zeros(6)},
+                  'x': {'value': np.random.random(6)}}},
+    'acos': {
+        'str': 'f=acos(x)',
+        'check_func': np.arccos,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6) - 0.5}}},
+    'arccos': {
+        'str': 'f=arccos(x)',
+        'check_func': np.arccos,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6) - 0.5}}},
+    'arccosh': {
+        'str': 'f=arccosh(x)',
+        'check_func': np.arccosh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': 1.1 + np.random.random(6)}}},
+    'acosh': {
+        'str': 'f=acosh(x)',
+        'check_func': np.arccosh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': 1.1 + np.random.random(6)}}},
+    'arange': {
+        'str': 'f=arange(0,10,2)',
+        'check_val': np.arange(0, 10, 2),
+        'args': {'f': {'value': np.zeros(5)}}},
+    'arcsin': {
+        'str': 'f=arcsin(x)',
+        'check_func': np.arcsin,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6) - .5}}},
+    'arcsinh': {
+        'str': 'f=arcsinh(x)',
+        'check_func': np.arcsinh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'asinh': {
+        'str': 'f=asinh(x)',
+        'check_func': np.arcsinh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'asin': {
+        'str': 'f=asin(x)',
+        'check_func': np.arcsin,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6) - .5}}},
+    'arctan': {
+        'str': 'f=arctan(x)',
+        'check_func': np.arctan,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'atan': {
+        'str': 'f=atan(x)',
+        'check_func': np.arctan,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'cos': {
+        'str': 'f=cos(x)',
+        'check_func': np.cos,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'cosh': {
+        'str': 'f=cosh(x)',
+        'check_func': np.cosh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'dot': {
+        'str': 'f=dot(x, y)',
+        'check_func': np.dot,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'e': {
+        'str': 'f=e',
+        'check_val': np.e,
+        'args': {'f': {'value': 0.0}}},
+    'erf': {
+        'str': 'f=erf(x)',
+        'check_func': scipy.special.erf,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'erfc': {
+        'str': 'f=erfc(x)',
+        'check_func': scipy.special.erfc,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'exp': {
+        'str': 'f=exp(x)',
+        'check_func': np.exp,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'expm1': {
+        'str': 'f=expm1(x)',
+        'check_func': np.expm1,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'factorial': {
+        'str': 'f=factorial(x)',
+        'check_func': scipy.special.factorial,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'fmax': {
+        'str': 'f=fmax(x, y)',
+        'check_func': np.fmax,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'fmin': {
+        'str': 'f=fmin(x, y)',
+        'check_func': np.fmin,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'inner': {
+        'str': 'f=inner(x, y)',
+        'check_func': np.inner,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'isinf': {
+        'str': 'f=isinf(x)',
+        'check_func': np.isinf,
+        'args': {'f': {'value': np.zeros(3)},
+                 'x': {'value': [0, np.inf, 5.0]}}},
+    'isnan': {
+        'str': 'f=isnan(x)',
+        'check_func': np.isnan,
+        'args': {'f': {'value': np.zeros(3)},
+                 'x': {'value': [0, np.nan, np.nan]}}},
+    'kron': {
+        'str': 'f=kron(x, y)',
+        'check_func': np.kron,
+        'args': {'f': {'value': np.zeros(36)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'linspace': {
+        'str': 'f=linspace(0,10,50)',
+        'check_val': np.linspace(0, 10, 50),
+        'args': {'f': {'value': np.zeros(50)}}},
+    'log': {
+        'str': 'f=log(x)',
+        'check_func': np.log,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6) + 0.1}}},
+    'log10': {
+        'str': 'f=log10(x)',
+        'check_func': np.log10,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6) + 0.1}}},
+    'log1p': {
+        'str': 'f=log1p(x)',
+        'check_func': np.log1p,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'matmul': {
+        'str': 'f=matmul(x, y)',
+        'check_func': np.matmul,
+        'args': {'f': {'value': np.zeros((3, 1))},
+                 'x': {'value': np.random.random((3, 3))},
+                 'y': {'value': np.random.random((3, 1))}}},
+    'maximum': {
+        'str': 'f=maximum(x, y)',
+        'check_func': np.maximum,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'minimum': {
+        'str': 'f=minimum(x, y)',
+        'check_func': np.minimum,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'ones': {
+        'str': 'f=ones(21)',
+        'check_val': np.ones(21),
+         'args': {'f': {'value': np.zeros(21)}}},
+    'outer': {
+        'str': 'f=outer(x, y)',
+        'check_func': np.outer,
+        'args': {'f': {'value': np.zeros((6, 6))},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6)}}},
+    'pi': {
+        'str': 'f=pi',
+        'check_val': np.pi,
+        'args': {'f': {'value': 0.0}}},
+    'power': {
+        'str': 'f=power(x, y)',
+        'check_func': np.power,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)},
+                 'y': {'value': np.random.random(6) + 1.0}}},
+    'prod': {
+        'str': 'f=prod(x)',
+        'check_func': np.prod,
+        'args': {'f': {'value': 0.0},
+                 'x': {'value': np.random.random(6)}}},
+    'sin': {
+        'str': 'f=sin(x)',
+        'check_func': np.sin,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'sinh': {
+        'str': 'f=sinh(x)',
+        'check_func': np.sinh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'sum': {
+        'str': 'f=sum(x)',
+        'check_func': np.sum,
+        'args': {'f': {'value': 0.0},
+                 'x': {'value': np.random.random(6)}}},
+    'tan': {
+        'str': 'f=tan(x)',
+        'check_func': np.tan,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'tanh': {
+        'str': 'f=tanh(x)',
+        'check_func': np.tanh,
+        'args': {'f': {'value': np.zeros(6)},
+                 'x': {'value': np.random.random(6)}}},
+    'tensordot': {
+        'str': 'f=tensordot(x, y)',
+        'check_func': np.tensordot,
+        'args': {'f': {'value': 0.0},
+                 'x': {'value': np.random.random((6, 6))},
+                 'y': {'value': np.random.random((6, 6))}}},
+    'zeros': {
+        'str': 'f=zeros(21)',
+        'check_val': np.zeros(21),
+        'args': {'f': {'value': np.zeros(21)}}},
+}
 
 
 class TestExecComp(unittest.TestCase):
@@ -330,6 +381,73 @@ class TestExecComp(unittest.TestCase):
 
         assert_rel_error(self, C1._outputs['y'], 4.0, 0.00001)
 
+    def test_units_varname(self):
+        prob = Problem(model=Group())
+
+        with self.assertRaises(TypeError) as cm:
+            prob.model.add_subsystem('C1', ExecComp('y=x+units+1.',
+                                                    x={'value': 2.0, 'units': 'm'},
+                                                    y={'units': 'm'},
+                                                    units=2.0))
+
+        self.assertEqual(str(cm.exception), "The units argument should be a str or None.")
+
+    def test_units_varname_str(self):
+        prob = Problem(model=Group())
+
+        with self.assertRaises(ValueError) as cm:
+            prob.model.add_subsystem('C1', ExecComp('y=x+units+1.',
+                                                    x={'value': 2.0, 'units': 'm'},
+                                                    y={'units': 'm'},
+                                                    units='two'))
+
+        self.assertEqual(str(cm.exception), "The units 'two' are invalid.")
+
+    def test_units_varname_novalue(self):
+        prob = Problem(model=Group())
+        prob.model.add_subsystem('indep', IndepVarComp('x', 100.0, units='cm'))
+        C1 = prob.model.add_subsystem('C1', ExecComp('y=x+units+1.',
+                                                     x={'value': 2.0, 'units': 'm'},
+                                                     y={'units': 'm'}))
+        prob.model.connect('indep.x', 'C1.x')
+
+        with self.assertRaises(NameError) as cm:
+            prob.setup(check=False)
+
+        self.assertEqual(str(cm.exception),
+                         "C1: cannot use variable name 'units' because it's a reserved keyword.")
+
+    def test_common_units(self):
+        # all variables in the ExecComp have the same units
+        prob = Problem(model=Group())
+        prob.model.add_subsystem('indep', IndepVarComp('x', 100.0, units='cm'))
+        C1 = prob.model.add_subsystem('C1', ExecComp('y=x+z+1.', units='m',
+                                                     x={'value': 2.0},
+                                                     z=2.0))
+        prob.model.connect('indep.x', 'C1.x')
+
+        prob.setup(check=False)
+
+        prob.set_solver_print(level=0)
+        prob.run_model()
+
+        assert_rel_error(self, C1._outputs['y'], 4.0, 0.00001)
+
+    def test_conflicting_units(self):
+        prob = Problem(model=Group())
+        prob.model.add_subsystem('indep', IndepVarComp('x', 100.0, units='cm'))
+        C1 = prob.model.add_subsystem('C1', ExecComp('y=x+z+1.', units='m',
+                                                     x={'value': 2.0, 'units': 'km'},
+                                                     z=2.0))
+        prob.model.connect('indep.x', 'C1.x')
+
+        with self.assertRaises(RuntimeError) as cm:
+            prob.setup(check=False)
+
+        self.assertEqual(str(cm.exception),
+                         "C1: units of 'km' have been specified for variable 'x', but "
+                         "units of 'm' have been specified for the entire component.")
+
     def test_math(self):
         prob = Problem(model=Group())
         C1 = prob.model.add_subsystem('C1', ExecComp('y=sin(x)', x=2.0))
@@ -678,8 +796,11 @@ class TestExecComp(unittest.TestCase):
 
         assert_rel_error(self, prob['comp.z'], 24.0, 0.00001)
 
+
+class TestExecCompParameterized(unittest.TestCase):
+
     @parameterized.expand(itertools.product([
-        func_name for func_name in _expr_dict if not func_name.startswith('_')
+      func_name for func_name in _expr_dict if not func_name.startswith('_')
     ]), name_func=lambda f, n, p: 'test_exec_comp_value_' + '_'.join(a for a in p.args))
     def test_exec_comp_value(self, f):
         test_data = _ufunc_test_data[f]
@@ -695,14 +816,12 @@ class TestExecComp(unittest.TestCase):
                 ivc.add_output(name=arg_name, val=arg_value['value'])
                 model.connect('ivc.{0}'.format(arg_name), 'comp.{0}'.format(arg_name))
 
-        model.add_subsystem('comp',
-                            ExecComp(test_data['str'], **test_data['args']),
+        model.add_subsystem('comp', ExecComp(test_data['str'], **test_data['args']),
                             promotes_outputs=['f'])
         prob.setup()
         prob.run_model()
 
         if 'check_func' in test_data:
-
             check_args = []
             try:
                 check_args.append(test_data['args']['x']['value'])
@@ -726,10 +845,9 @@ class TestExecComp(unittest.TestCase):
                 print(f, 'does not support complex-step differentiation')
 
     @parameterized.expand(itertools.product([
-        func_name for func_name in _expr_dict if not func_name.startswith('_')
+      func_name for func_name in _expr_dict if not func_name.startswith('_')
     ]), name_func=lambda f, n, p: 'test_exec_comp_jac_' + '_'.join(a for a in p.args))
     def test_exec_comp_jac(self, f):
-
         test_data = _ufunc_test_data[f]
 
         prob = Problem()
