@@ -38,6 +38,7 @@ class ExplicitComponent(Component):
 
         self._inst_functs = {name: getattr(self, name, None) for name in _inst_functs}
         self._has_compute_partials = overrides_method('compute_partials', self, ExplicitComponent)
+        self.options.undeclare('assembled_jac_type')
 
     def _configure(self):
         """
@@ -87,6 +88,18 @@ class ExplicitComponent(Component):
             self._declare_partials(out_name, out_name, rows=arange, cols=arange,
                                    val=np.full(meta['size'], -1.))
 
+    def _setup_jacobians(self, recurse=True):
+        """
+        Set and populate jacobian.
+
+        Parameters
+        ----------
+        recurse : bool
+            If True, setup jacobians in all descendants. (ignored)
+        """
+        if self._use_derivatives:
+            self._set_partials_meta()
+
     def add_output(self, name, val=1.0, shape=None, units=None, res_units=None, desc='',
                    lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None):
         """
@@ -192,15 +205,6 @@ class ExplicitComponent(Component):
     def _solve_nonlinear(self):
         """
         Compute outputs. The model is assumed to be in a scaled state.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
         super(ExplicitComponent, self)._solve_nonlinear()
 
@@ -210,15 +214,13 @@ class ExplicitComponent(Component):
                 self._inputs.read_only = True
                 try:
                     if self._discrete_inputs or self._discrete_outputs:
-                        failed = self.compute(self._inputs, self._outputs, self._discrete_inputs,
-                                              self._discrete_outputs)
+                        self.compute(self._inputs, self._outputs, self._discrete_inputs,
+                                     self._discrete_outputs)
                     else:
-                        failed = self.compute(self._inputs, self._outputs)
+                        self.compute(self._inputs, self._outputs)
                 finally:
                     self._inputs.read_only = False
 
-        return bool(failed), 0., 0.
-
     def _apply_linear(self, jac, vec_names, rel_systems, mode, scope_out=None, scope_in=None):
         """
         Compute jac-vec product. The model is assumed to be in a scaled state.
@@ -304,14 +306,6 @@ class ExplicitComponent(Component):
         rel_systems : set of str
             Set of names of relevant systems based on the current linear solve.
 
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
         for vec_name in vec_names:
             if vec_name in self._rel_vec_names:
@@ -340,8 +334,6 @@ class ExplicitComponent(Component):
                     # ExplicitComponent jacobian defined with -1 on diagonal.
                     d_residuals *= -1.0
 
-        return False, 0., 0.
-
     def _linearize(self, jac=None, sub_do_ln=False):
         """
         Compute jacobian / factorization. The model is assumed to be in a scaled state.
@@ -389,11 +381,6 @@ class ExplicitComponent(Component):
             If not None, dict containing discrete input values.
         discrete_outputs : dict or None
             If not None, dict containing discrete output values.
-
-        Returns
-        -------
-        bool or None
-            None or False if run successfully; True if there was a failure.
         """
         pass
 
