@@ -3,7 +3,6 @@ from __future__ import print_function
 import os
 import shutil
 import tempfile
-import warnings
 
 import unittest
 import numpy as np
@@ -18,13 +17,14 @@ except ImportError:
     load_npz = None
 
 from openmdao.api import Problem, IndepVarComp, ExecComp, DirectSolver,\
-    ExplicitComponent, LinearRunOnce, ScipyOptimizeDriver, ParallelGroup, Group
-from openmdao.utils.assert_utils import assert_rel_error
-
+    ExplicitComponent, LinearRunOnce, ScipyOptimizeDriver, ParallelGroup, Group, \
+    SqliteRecorder, CaseReader
+from openmdao.utils.assert_utils import assert_rel_error, assert_warning
 from openmdao.utils.general_utils import set_pyoptsparse_opt
 from openmdao.utils.coloring import get_simul_meta, _solves_info
 from openmdao.utils.mpi import MPI
 from openmdao.test_suite.tot_jac_builder import TotJacBuilder
+
 import openmdao.test_suite
 
 try:
@@ -53,7 +53,7 @@ class CounterGroup(Group):
 SIZE = 10
 
 def run_opt(driver_class, mode, assemble_type=None, color_info=None, sparsity=None, derivs=True,
-           **options):
+            recorder=None, **options):
 
     p = Problem(model=CounterGroup())
 
@@ -127,6 +127,9 @@ def run_opt(driver_class, mode, assemble_type=None, color_info=None, sparsity=No
     elif sparsity is not None:
         p.driver.set_total_jac_sparsity(sparsity)
 
+    if recorder:
+        p.driver.add_recorder(recorder)
+
     p.setup(mode=mode, derivatives=derivs)
     p.run_driver()
 
@@ -354,6 +357,37 @@ class SimulColoringPyoptSparseTestCase(unittest.TestCase):
                          (p_color.model._solve_count - 21 * 4) / 5)
 
 
+@unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
+class SimulColoringRecordingTestCase(unittest.TestCase):
+
+    def setUp(self):
+        from tempfile import mkdtemp
+        self.dir = mkdtemp()
+        self.original_path = os.getcwd()
+        os.chdir(self.dir)
+
+    def tearDown(self):
+        os.chdir(self.original_path)
+        try:
+            shutil.rmtree(self.dir)
+        except OSError as e:
+            # If directory already deleted, keep going
+            if e.errno not in (errno.ENOENT, errno.EACCES, errno.EPERM):
+                raise e
+
+    def test_recording(self):
+        # coloring involves an underlying call to run_model (and final_setup),
+        # this verifies that it is handled properly by the recording setup logic
+        recorder = SqliteRecorder('cases.sql')
+
+        p = run_opt(pyOptSparseDriver, 'auto', assemble_type='csc', optimizer='SNOPT',
+                    dynamic_simul_derivs=True, print_results=False, recorder=recorder)
+
+        cr = CaseReader('cases.sql')
+
+        self.assertEqual(cr.list_cases(), ['rank0:SNOPT|%d' % i for i in range(p.driver.iter_count)])
+
+
 class SimulColoringPyoptSparseRevTestCase(unittest.TestCase):
     """Reverse coloring tests for pyoptsparse."""
 
@@ -613,11 +647,15 @@ class SimulColoringScipyTestCase(unittest.TestCase):
         # check for proper handling if someone calls compute_totals on Problem with different set or different order
         # of desvars/responses than were used to define the coloring.  Behavior should be that coloring is turned off
         # and a warning is issued.
-        with warnings.catch_warnings(record=True) as w:
+        msg = "compute_totals called using a different list of design vars and/or responses than those used " \
+              "to define coloring, so coloring will be turned off.\ncoloring design vars: " \
+              "['indeps.x', 'indeps.y', 'indeps.r'], current design vars: ['indeps.x', 'indeps.y', 'indeps.r']\n" \
+              "coloring responses: ['circle.area', 'r_con.g', 'theta_con.g', 'delta_theta_con.g', 'l_conx.g'], " \
+              "current responses: ['delta_theta_con.g', 'circle.area', 'r_con.g', 'theta_con.g', 'l_conx.g']."
+
+        with assert_warning(UserWarning, msg):
             p_color.compute_totals(of=['delta_theta_con.g', 'circle.area', 'r_con.g', 'theta_con.g', 'l_conx.g'],
                                    wrt=['x', 'y', 'r'])
-        self.assertEqual(len(w), 1)
-        self.assertEqual(str(w[0].message), "compute_totals called using a different list of design vars and/or responses than those used to define coloring, so coloring will be turned off.\ncoloring design vars: ['indeps.x', 'indeps.y', 'indeps.r'], current design vars: ['indeps.x', 'indeps.y', 'indeps.r']\ncoloring responses: ['circle.area', 'r_con.g', 'theta_con.g', 'delta_theta_con.g', 'l_conx.g'], current responses: ['delta_theta_con.g', 'circle.area', 'r_con.g', 'theta_con.g', 'l_conx.g'].")
 
     def test_bad_mode(self):
         with self.assertRaises(Exception) as context:
