@@ -290,10 +290,9 @@ class TestScaling(unittest.TestCase):
             prob.set_solver_print(level=0)
 
             prob.setup(check=False)
-            result = prob.run_model()
+            prob.run_model()
 
-            success = not result[0]
-            return success
+            return np.linalg.norm(prob.model._residuals._data) < 1e-5
 
         # ---------------------------
         # coeffs: r1, r2, c1, c2
@@ -373,6 +372,7 @@ class TestScaling(unittest.TestCase):
 
         model.nonlinear_solver = NonlinearBlockGS()
         model.nonlinear_solver.options['maxiter'] = 1
+        model.nonlinear_solver.options['use_apply_nonlinear'] = True
 
         prob.set_solver_print(level=0)
 
@@ -411,6 +411,7 @@ class TestScaling(unittest.TestCase):
 
         model.nonlinear_solver = NonlinearBlockGS()
         model.nonlinear_solver.options['maxiter'] = 1
+        model.nonlinear_solver.options['use_apply_nonlinear'] = True
 
         prob.set_solver_print(level=0)
 
@@ -445,6 +446,7 @@ class TestScaling(unittest.TestCase):
 
         model.nonlinear_solver = NonlinearBlockGS()
         model.nonlinear_solver.options['maxiter'] = 1
+        model.nonlinear_solver.options['use_apply_nonlinear'] = True
 
         prob.set_solver_print(level=0)
 
@@ -481,6 +483,7 @@ class TestScaling(unittest.TestCase):
 
         model.nonlinear_solver = NonlinearBlockGS()
         model.nonlinear_solver.options['maxiter'] = 1
+        model.nonlinear_solver.options['use_apply_nonlinear'] = True
 
         prob.set_solver_print(level=0)
 
