@@ -12,6 +12,9 @@ function PtN2Diagram(parentDiv, modelData) {
     var transitionStartDelay = DEFAULT_TRANSITION_START_DELAY;
     var idCounter = 0;
     var maxDepth = 1;
+
+    var maxSystemDepth = 1; // For use with the right hand side solver tree. Only want max depth of solvers, not params,..
+
     var RIGHT_TEXT_MARGIN_PX = 8; // How much space in px (left and) right of text in partition tree
 
     //N^2 vars
@@ -54,7 +57,8 @@ function PtN2Diagram(parentDiv, modelData) {
         .attr("class", "arrowHead");
 
     setN2Group();
-    var pTreeGroup = svg.append("g");
+    var pTreeGroup = svg.append("g").attr("id", "tree"); // id given just so it is easier to see in Chrome dev tools when debugging
+    var pSolverTreeGroup = svg.append("g").attr("id", "solver_tree");
 
     function updateRootTypes() {
         if (!showParams) return;
@@ -276,6 +280,11 @@ function PtN2Diagram(parentDiv, modelData) {
             ky0 = ky;
             xScalerPTree0 = xScalerPTree.copy();
             yScalerPTree0 = yScalerPTree.copy();
+
+            kxSolver0 = kxSolver;
+            kySolver0 = kySolver;
+            xScalerPSolverTree0 = xScalerPSolverTree.copy();
+            yScalerPSolverTree0 = yScalerPSolverTree.copy();
         }
 
         kx = (zoomedElement.x ? widthPTreePx - PARENT_NODE_WIDTH_PX : widthPTreePx) / (1 - zoomedElement.x);
@@ -283,33 +292,49 @@ function PtN2Diagram(parentDiv, modelData) {
         xScalerPTree.domain([zoomedElement.x, 1]).range([zoomedElement.x ? PARENT_NODE_WIDTH_PX : 0, widthPTreePx]);
         yScalerPTree.domain([zoomedElement.y, zoomedElement.y + zoomedElement.height]).range([0, HEIGHT_PX]);
 
+        kxSolver = (zoomedElement.xSolver ? widthPSolverTreePx - PARENT_NODE_WIDTH_PX : widthPSolverTreePx) / (1 - zoomedElement.xSolver);
+        kySolver = HEIGHT_PX / zoomedElement.heightSolver;
+        xScalerPSolverTree.domain([zoomedElement.xSolver, 1]).range([zoomedElement.xSolver ? PARENT_NODE_WIDTH_PX : 0, widthPSolverTreePx]);
+        yScalerPSolverTree.domain([zoomedElement.ySolver, zoomedElement.ySolver + zoomedElement.heightSolver]).range([0, HEIGHT_PX]);
+
         if (xScalerPTree0 == null) { //first run.. duplicate
             kx0 = kx;
             ky0 = ky;
             xScalerPTree0 = xScalerPTree.copy();
             yScalerPTree0 = yScalerPTree.copy();
 
+            kxSolver0 = kxSolver;
+            kySolver0 = kySolver;
+            xScalerPSolverTree0 = xScalerPSolverTree.copy();
+            yScalerPSolverTree0 = yScalerPSolverTree.copy();
+
             //Update svg dimensions before ComputeLayout() changes widthPTreePx
-            svgDiv.style("width", (widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + 2 * SVG_MARGIN) + "px")
+            svgDiv.style("width", (widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + widthPSolverTreePx + 2 * SVG_MARGIN + PTREE_N2_GAP_PX) + "px")
                   .style("height", (HEIGHT_PX + 2 * SVG_MARGIN) + "px");
-            svg.attr("width", widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + 2 * SVG_MARGIN)
+            svg.attr("width", widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + widthPSolverTreePx + 2 * SVG_MARGIN + PTREE_N2_GAP_PX)
                .attr("height", HEIGHT_PX + 2 * SVG_MARGIN);
+
             n2Group.attr("transform", "translate(" + (widthPTreePx + PTREE_N2_GAP_PX + SVG_MARGIN) + "," + SVG_MARGIN + ")");
             pTreeGroup.attr("transform", "translate(" + SVG_MARGIN + "," + SVG_MARGIN + ")");
+
+            pSolverTreeGroup.attr("transform", "translate(" + (widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + SVG_MARGIN + PTREE_N2_GAP_PX) + "," + SVG_MARGIN + ")");
         }
 
         sharedTransition = d3.transition().duration(TRANSITION_DURATION).delay(transitionStartDelay); //do this after intense computation
         transitionStartDelay = DEFAULT_TRANSITION_START_DELAY;
 
         //Update svg dimensions with transition after ComputeLayout() changes widthPTreePx
-        svgDiv.transition(sharedTransition).style("width", (widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + 2 * SVG_MARGIN) + "px")
+        svgDiv.transition(sharedTransition).style("width", (widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + widthPSolverTreePx + 2 * SVG_MARGIN + PTREE_N2_GAP_PX) + "px")
             .style("height", (HEIGHT_PX + 2 * SVG_MARGIN) + "px");
-        svg.transition(sharedTransition).attr("width", widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + 2 * SVG_MARGIN)
+        svg.transition(sharedTransition).attr("width", widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + widthPSolverTreePx + 2 * SVG_MARGIN + PTREE_N2_GAP_PX)
             .attr("height", HEIGHT_PX + 2 * SVG_MARGIN);
+
         n2Group.transition(sharedTransition).attr("transform", "translate(" + (widthPTreePx + PTREE_N2_GAP_PX + SVG_MARGIN) + "," + SVG_MARGIN + ")");
         pTreeGroup.transition(sharedTransition).attr("transform", "translate(" + SVG_MARGIN + "," + SVG_MARGIN + ")");
         n2BackgroundRect.transition(sharedTransition).attr("width", WIDTH_N2_PX).attr("height", HEIGHT_PX);
 
+        pSolverTreeGroup.transition(sharedTransition).attr("transform", "translate(" + (widthPTreePx + PTREE_N2_GAP_PX + WIDTH_N2_PX + SVG_MARGIN + PTREE_N2_GAP_PX) + "," + SVG_MARGIN + ")");
+
         var sel = pTreeGroup.selectAll(".partition_group")
             .data(d3NodesArray, function (d) {
                 return d.id;
@@ -422,6 +447,137 @@ function PtN2Diagram(parentDiv, modelData) {
             .style("opacity", 0);
 
 
+       var selSolver = pSolverTreeGroup.selectAll(".solver_group")
+            .data(d3SolverNodesArray, function (d) {
+                return d.id;
+            });
+
+        function getSolverClass(showLinearSolverNames, linear_solver_name, nonlinear_solver_name){
+            if (showLinearSolverNames){
+                if (linearSolverNames.indexOf(linear_solver_name) >= 0){
+                    solver_class = linearSolverClasses[linear_solver_name]
+                } else {
+                    solver_class = linearSolverClasses["other"]; // user must have defined their own solver that we do not know about
+                }
+            } else {
+                if (nonLinearSolverNames.indexOf(nonlinear_solver_name) >= 0){
+                    solver_class = nonLinearSolverClasses[nonlinear_solver_name]
+                } else {
+                    solver_class = nonLinearSolverClasses["other"]; // user must have defined their own solver that we do not know about
+                }
+            }
+            return solver_class;
+        }
+
+        var nodeSolverEnter = selSolver.enter().append("svg:g")
+            .attr("class", function (d) {
+                solver_class = getSolverClass(showLinearSolverNames, d.linear_solver, d.nonlinear_solver) ;
+                return solver_class + " " + "solver_group " + GetClass(d) ;
+            })
+            .attr("transform", function (d) {
+                x = 1.0 - d.xSolver0 - d.widthSolver0; // The magic for reversing the blocks on the right side
+                // The solver tree goes from the root on the right and expands to the left
+                return "translate(" + xScalerPSolverTree0(x) + "," + yScalerPSolverTree0(d.ySolver0) + ")";
+            })
+            .on("click", function (d) { LeftClick(d, this); })
+            .on("contextmenu", function (d) { RightClick(d, this); })
+            .on("mouseover", function (d) {
+                if (abs2prom != undefined) {
+                    if (d.type == "param" || d.type == "unconnected_param") {
+                        return tooltip.text(abs2prom.input[d.absPathName])
+                                      .style("visibility", "visible");
+                    }
+                    if (d.type == "unknown") {
+                        return tooltip.text(abs2prom.output[d.absPathName])
+                                      .style("visibility", "visible");
+                    }
+                }
+            })
+            .on("mouseleave", function (d) {
+                if (abs2prom != undefined) {
+                    return tooltip.style("visibility", "hidden");
+                }
+            })
+            .on("mousemove", function(){
+                if (abs2prom != undefined) {
+                    return tooltip.style("top", (d3.event.pageY-30)+"px")
+                                  .style("left",(d3.event.pageX+5)+"px");
+                }
+            });
+
+        nodeSolverEnter.append("svg:rect")
+            .attr("width", function (d) {
+                return d.widthSolver0 * kxSolver0;//0;//
+            })
+            .attr("height", function (d) {
+                return d.heightSolver0 * kySolver0;
+            });
+
+        nodeSolverEnter.append("svg:text")
+            .attr("dy", ".35em")
+            .attr("transform", function (d) {
+                var anchorX = d.widthSolver0 * kxSolver0 - RIGHT_TEXT_MARGIN_PX;
+                return "translate(" + anchorX + "," + d.heightSolver0 * kySolver0 / 2 + ")";
+            })
+            .style("opacity", function (d) {
+                if (d.depth < zoomedElement.depth) return 0;
+                return d.textOpacity0;
+            })
+            .text(GetSolverText);
+
+        var nodeSolverUpdate = nodeSolverEnter.merge(selSolver).transition(sharedTransition)
+            .attr("class", function (d) {
+                solver_class = getSolverClass(showLinearSolverNames, d.linear_solver, d.nonlinear_solver) ;
+                return solver_class + " " + "solver_group " + GetClass(d) ;
+            })
+            .attr("transform", function (d) {
+                x = 1.0 - d.xSolver - d.widthSolver; // The magic for reversing the blocks on the right side
+                return "translate(" + xScalerPSolverTree(x) + "," + yScalerPSolverTree(d.ySolver) + ")";
+            });
+
+        nodeSolverUpdate.select("rect")
+            .attr("width", function (d) {
+                return d.widthSolver * kxSolver;
+            })
+            .attr("height", function (d) {
+                return d.heightSolver * kySolver;
+            });
+
+        nodeSolverUpdate.select("text")
+            .attr("transform", function (d) {
+                var anchorX = d.widthSolver * kxSolver - RIGHT_TEXT_MARGIN_PX;
+                return "translate(" + anchorX + "," + d.heightSolver * kySolver / 2 + ")";
+            })
+            .style("opacity", function (d) {
+                if (d.depth < zoomedElement.depth) return 0;
+                return d.textOpacity;
+            })
+            .text(GetSolverText);
+
+
+        // Transition exiting nodes to the parent's new position.
+        var nodeSolverExit = selSolver.exit().transition(sharedTransition)
+            .attr("transform", function (d) {
+                return "translate(" + xScalerPSolverTree(d.xSolver) + "," + yScalerPSolverTree(d.ySolver) + ")";
+            })
+            .remove();
+
+        nodeSolverExit.select("rect")
+            .attr("width", function (d) {
+                return d.widthSolver * kxSolver;//0;//
+            })
+            .attr("height", function (d) {
+                return d.heightSolver * kySolver;
+            });
+
+        nodeSolverExit.select("text")
+            .attr("transform", function (d) {
+                var anchorX = d.widthSolver * kxSolver - RIGHT_TEXT_MARGIN_PX;
+                return "translate(" + anchorX + "," + d.heightSolver * kySolver / 2 + ")";
+            })
+            .style("opacity", 0);
+
+
         ClearArrowsAndConnects()
         DrawMatrix();
     }
@@ -518,6 +674,11 @@ function PtN2Diagram(parentDiv, modelData) {
         return retVal;
     }
 
+    function GetSolverText(d) {
+        var retVal = showLinearSolverNames ? d.linear_solver : d.nonlinear_solver;
+        return retVal;
+    }
+
     //Sets parents, depth, and nameWidthPx of all nodes.  Also finds and sets maxDepth.
     function InitTree(d, parent, depth) {
         d.numLeaves = 0; //for nested params
@@ -548,6 +709,11 @@ function PtN2Diagram(parentDiv, modelData) {
             }
         }
         maxDepth = Math.max(depth, maxDepth);
+
+        if (d.type === "subsystem") {
+            maxSystemDepth = Math.max(depth, maxSystemDepth);
+        }
+
         if (d.children) {
             for (var i = 0; i < d.children.length; ++i) {
                 var implicit = InitTree(d.children[i], d, depth + 1);
@@ -564,6 +730,9 @@ function PtN2Diagram(parentDiv, modelData) {
         var columnWidthsPx = new Array(maxDepth + 1).fill(0.0),// since depth is one based
             columnLocationsPx = new Array(maxDepth + 1).fill(0.0);
 
+        var columnSolverWidthsPx = new Array(maxDepth + 1).fill(0.0),// since depth is one based
+            columnSolverLocationsPx = new Array(maxDepth + 1).fill(0.0);
+
         var textWidthGroup = svg.append("svg:g").attr("class", "partition_group");
         var textWidthText = textWidthGroup.append("svg:text")
             .text("")
@@ -617,6 +786,16 @@ function PtN2Diagram(parentDiv, modelData) {
             }
         }
 
+        function UpdateSolverTextWidths(d) {
+            if ((d.type === "param" || d.type === "unconnected_param") || d.varIsHidden) return;
+            d.nameSolverWidthPx = GetTextWidth(GetSolverText(d)) + 2 * RIGHT_TEXT_MARGIN_PX;
+            if (d.children) {
+                for (var i = 0; i < d.children.length; ++i) {
+                    UpdateSolverTextWidths(d.children[i]);
+                }
+            }
+        }
+
         function ComputeColumnWidths(d) {
             var greatestDepth = 0;
             var leafWidthsPx = new Array(maxDepth + 1).fill(0.0);
@@ -660,6 +839,47 @@ function PtN2Diagram(parentDiv, modelData) {
 
         }
 
+        function ComputeSolverColumnWidths(d) {
+            var greatestDepth = 0;
+            var leafSolverWidthsPx = new Array(maxDepth + 1).fill(0.0);
+
+            function DoComputeSolverColumnWidths(d) {
+                var heightPx = HEIGHT_PX * d.numSolverLeaves / zoomedElement.numSolverLeaves;
+                d.textOpacity0 = d.hasOwnProperty('textOpacity') ? d.textOpacity : 0;
+                d.textOpacity = (heightPx > FONT_SIZE_PX) ? 1 : 0;
+                var hasVisibleDetail = (heightPx >= 2.0);
+                var widthPx = 1e-3;
+                if (hasVisibleDetail) widthPx = MIN_COLUMN_WIDTH_PX;
+                if (d.textOpacity > 0.5) widthPx = d.nameSolverWidthPx;
+
+                greatestDepth = Math.max(greatestDepth, d.depth);
+
+                if (d.subsystem_children && !d.isMinimized) { //not leaf
+                    columnSolverWidthsPx[d.depth] = Math.max(columnSolverWidthsPx[d.depth], widthPx);
+                    for (var i = 0; i < d.subsystem_children.length; ++i) {
+                        DoComputeSolverColumnWidths(d.subsystem_children[i]);
+                    }
+                }
+                else { //leaf
+                    leafSolverWidthsPx[d.depth] = Math.max(leafSolverWidthsPx[d.depth], widthPx);
+                }
+            }
+
+            DoComputeSolverColumnWidths(d);
+
+
+            var sum = 0;
+            var lastColumnWidth = 0;
+            for (var i = leafSolverWidthsPx.length - 1; i >= zoomedElement.depth; --i) {
+                sum += columnSolverWidthsPx[i];
+                var lastWidthNeeded = leafSolverWidthsPx[i] - sum;
+                lastColumnWidth = Math.max(lastWidthNeeded, lastColumnWidth);
+            }
+            columnSolverWidthsPx[zoomedElement.depth - 1] = PARENT_NODE_WIDTH_PX;
+            columnSolverWidthsPx[greatestDepth] = lastColumnWidth;
+
+        }
+
 
         function ComputeLeaves(d) {
             if ((!showParams && (d.type === "param" || d.type === "unconnected_params")) || d.varIsHidden) {
@@ -676,6 +896,22 @@ function PtN2Diagram(parentDiv, modelData) {
             }
         }
 
+        function ComputeSolverLeaves(d) {
+            if ((!showParams && (d.type === "param" || d.type === "unconnected_params")) || d.varIsHidden) {
+                d.numSolverLeaves = 0;
+                return;
+            }
+
+            var doRecurse = d.children && !d.isMinimized;
+            d.numSolverLeaves = doRecurse ? 0 : 1; //no children: init to 0 because will be added later
+            if (!doRecurse) return;
+
+            for (var i = 0; i < d.children.length; ++i) {
+                ComputeSolverLeaves(d.children[i]);
+                d.numSolverLeaves += d.children[i].numSolverLeaves;
+            }
+        }
+
         function ComputeNormalizedPositions(d, leafCounter, isChildOfZoomed, earliestMinimizedParent) {
             isChildOfZoomed = (isChildOfZoomed) ? true : (d === zoomedElement);
             if (earliestMinimizedParent == null && isChildOfZoomed) {
@@ -714,14 +950,87 @@ function PtN2Diagram(parentDiv, modelData) {
         }
 
 
+        function ComputeSolverNormalizedPositions(d, leafCounter, isChildOfZoomed, earliestMinimizedParent) {
+            isChildOfZoomed = (isChildOfZoomed) ? true : (d === zoomedElement);
+            if (earliestMinimizedParent == null && isChildOfZoomed) {
+                if (d.type === "subsystem" || d.type === "root" ) d3SolverNodesArray.push(d);
+                if (!d.children || d.isMinimized) { //at a "leaf" node
+                    if ((d.type !== "param" && d.type !== "unconnected_param") && !d.varIsHidden) d3SolverRightTextNodesArrayZoomed.push(d);
+                    earliestMinimizedParent = d;
+                }
+            }
+            var node = (earliestMinimizedParent) ? earliestMinimizedParent : d;
+            d.rootIndex0 = d.hasOwnProperty('rootIndex') ? d.rootIndex : leafCounter;
+            d.xSolver0 = d.hasOwnProperty('xSolver') ? d.xSolver : 1e-6;
+            d.ySolver0 = d.hasOwnProperty('ySolver') ? d.ySolver : 1e-6;
+            d.widthSolver0 = d.hasOwnProperty('widthSolver') ? d.widthSolver : 1e-6;
+            d.heightSolver0 = d.hasOwnProperty('heightSolver') ? d.heightSolver : 1e-6;
+            d.xSolver = columnSolverLocationsPx[node.depth] / widthPSolverTreePx;
+            d.ySolver = leafCounter / root.numSolverLeaves;
+            d.widthSolver = (d.subsystem_children && !d.isMinimized) ? (columnSolverWidthsPx[node.depth] / widthPSolverTreePx) : 1 - node.xSolver;//1-d.x;
+
+            d.heightSolver = node.numSolverLeaves / root.numSolverLeaves; 111
+
+
+
+
+
+
+            if ((!showParams && (d.type === "param" || d.type === "unconnected_param")) || d.varIsHidden) { //param or hidden leaf leaving
+                d.xSolver = columnLocationsPx[d.parentComponent.depthqqq + 1] / widthPTreePx;
+                d.ySolver = d.parentComponent.y;
+                d.widthSolver = 1e-6;
+                d.heightSolver = 1e-6;
+            }
+
+
+
+
+
+
+            if (d.children) {
+                for (var i = 0; i < d.children.length; ++i) {
+                    ComputeSolverNormalizedPositions(d.children[i], leafCounter, isChildOfZoomed, earliestMinimizedParent);
+                    if (earliestMinimizedParent == null) { //numleaves is only valid passed nonminimized nodes
+                        leafCounter += d.children[i].numSolverLeaves;
+                    }
+                }
+            }
+        }
 
         UpdateTextWidths(zoomedElement);
+
+        UpdateSolverTextWidths(zoomedElement);
+
         ComputeLeaves(root);
+
+        ComputeSolverLeaves(root);
+
         d3NodesArray = [];
         d3RightTextNodesArrayZoomed = [];
 
-        //COLUMN WIDTH COMPUTATION
+        d3SolverNodesArray = [];
+        d3SolverRightTextNodesArrayZoomed = [];
+
         ComputeColumnWidths(zoomedElement);
+
+        function InitSubSystemChildren(d){
+            for (var i = 0; i < d.children.length; ++i) {
+                var child = d.children[i];
+                if (child.type === "subsystem"){
+                    if (!d.hasOwnProperty("subsystem_children")) {
+                        d.subsystem_children = [];
+                    }
+                    d.subsystem_children.push(child);
+                    InitSubSystemChildren(child);
+                }
+            }
+        }
+
+        InitSubSystemChildren(root);
+
+        ComputeSolverColumnWidths(zoomedElement);
+
         // Now the column_width array is relative to the zoomedElement
         //    and the computation of the widths only includes visible items after the zoom
         widthPTreePx = 0;
@@ -730,11 +1039,22 @@ function PtN2Diagram(parentDiv, modelData) {
             widthPTreePx += columnWidthsPx[depth];
         }
 
+        widthPSolverTreePx = 0;
+        for (var depth = 1; depth <= maxDepth; ++depth) {
+            columnSolverLocationsPx[depth] = widthPSolverTreePx;
+            widthPSolverTreePx += columnSolverWidthsPx[depth];
+        }
+
         ComputeNormalizedPositions(root, 0, false, null);
         if (zoomedElement.parent) {
             d3NodesArray.push(zoomedElement.parent);
         }
 
+        ComputeSolverNormalizedPositions(root, 0, false, null);
+        if (zoomedElement.parent) {
+            d3SolverNodesArray.push(zoomedElement.parent);
+        }
+
         if (updateRecomputesAutoComplete) {
             autoCompleteListNames = [];
             autoCompleteListPathNames = [];
@@ -1518,6 +1838,10 @@ function PtN2Diagram(parentDiv, modelData) {
             var newText = (i == height) ? ("<b>" + i + "px</b>") : (i + "px");
             parentDiv.querySelector("#idVerticalResize" + i + "px").innerHTML = newText;
         }
+        for (var i = 2000; i <= 4000; i += 1000) {
+            var newText = (i == height) ? ("<b>" + i + "px</b>") : (i + "px");
+            parentDiv.querySelector("#idVerticalResize" + i + "px").innerHTML = newText;
+        }
         ClearArrowsAndConnects();
         HEIGHT_PX = height;
         LEVEL_OF_DETAIL_THRESHOLD = HEIGHT_PX / 3;
@@ -1530,7 +1854,7 @@ function PtN2Diagram(parentDiv, modelData) {
     function ShowParamsCheckboxChange() {
         if (zoomedElement.type === "param" || zoomedElement.type === "unconnected_param") return;
         showParams = !showParams;
-        parentDiv.querySelector("#showParamsButtonId").className = showParams ? "myButton myButtonToggledOn" : "myButton";
+        parentDiv.querySelector("#showParamsButtonId").className = !showParams ? "myButton myButtonToggledOn" : "myButton";
 
         FindRootOfChangeFunction = FindRootOfChangeForShowParams;
         lastClickWasLeft = false;
@@ -1540,6 +1864,13 @@ function PtN2Diagram(parentDiv, modelData) {
         Update();
     }
 
+    function ToggleSolverNamesCheckboxChange() {
+        showLinearSolverNames = !showLinearSolverNames;
+        parentDiv.querySelector("#toggleSolverNamesButtonId").className = !showLinearSolverNames ? "myButton myButtonToggledOn" : "myButton";
+        SetupLegend(d3, d3ContentDiv);
+        Update();
+    }
+
     function ShowPathCheckboxChange() {
         showPath = !showPath;
         parentDiv.querySelector("#currentPathId").style.display = showPath ? "block" : "none";
@@ -1571,6 +1902,8 @@ function PtN2Diagram(parentDiv, modelData) {
         div.querySelector("#showLegendButtonId").onclick = ToggleLegend;
         div.querySelector("#showParamsButtonId").onclick = ShowParamsCheckboxChange;
 
+        div.querySelector("#toggleSolverNamesButtonId").onclick = ToggleSolverNamesCheckboxChange;
+
         for (var i = 8; i <= 14; ++i) {
             var f = function (idx) {
                 return function () { FontSizeSelectChange(idx); };
@@ -1584,6 +1917,12 @@ function PtN2Diagram(parentDiv, modelData) {
             }(i);
             div.querySelector("#idVerticalResize" + i + "px").onclick = f;
         }
+        for (var i = 2000; i <= 4000; i += 1000) {
+            var f = function(idx) {
+                return function () { VerticalResize(idx); };
+            }(i);
+            div.querySelector("#idVerticalResize" + i + "px").onclick = f;
+        }
 
         div.querySelector("#saveSvgButtonId").onclick = function () { SaveSvg(parentDiv) };
         div.querySelector("#helpButtonId").onclick = DisplayModal;
@@ -1605,5 +1944,16 @@ var mouseClickN2;
 var hasInputConn;
 var treeData, connectionList;
 modelData.tree.name = 'model'; //Change 'root' to 'model'
+function ChangeBlankSolverToNone(d) {
+    if (d.linear_solver === "") d.linear_solver = "None";
+    if (d.nonlinear_solver === "") d.nonlinear_solver = "None";
+    if (d.children) {
+        for (var i = 0; i < d.children.length; ++i) {
+            ChangeBlankSolverToNone(d.children[i]);
+        }
+    }
+}
+ChangeBlankSolverToNone(modelData.tree);
+
 var app = PtN2Diagram(document.getElementById("ptN2ContentDivId"), modelData);
 
