@@ -4,9 +4,7 @@
 import sys
 import os
 import importlib
-import textwrap
 
-from numpydoc.docscrape import NumpyDocString, Reader
 from mock import Mock
 
 from openmdao.docs.config_params import MOCK_MODULES
@@ -18,7 +16,7 @@ for mod_name in MOCK_MODULES:
     try:
         importlib.import_module(mod_name)
     except ImportError:
-        sys.modules[mod_name]=Mock()
+        sys.modules[mod_name] = Mock()
 
 # start off running the monkeypatch to keep options/parameters
 # usable in docstring for autodoc.
