@@ -38,12 +38,20 @@ optlist = ['ALPSO', 'CONMIN', 'FSQP', 'IPOPT', 'NLPQLP',
 # All optimizers that require an initial run
 run_required = ['NSGA2', 'ALPSO']
 
-CITATIONS = """
-@phdthesis{hwang_thesis_2015,
-  author       = {John T. Hwang},
-  title        = {A Modular Approach to Large-Scale Design Optimization of Aerospace Systems},
-  school       = {University of Michigan},
-  year         = 2015
+CITATIONS = """@article{Hwang_maud_2018
+ author = {Hwang, John T. and Martins, Joaquim R.R.A.},
+ title = "{A Computational Architecture for Coupling Heterogeneous
+          Numerical Models and Computing Coupled Derivatives}",
+ journal = "{ACM Trans. Math. Softw.}",
+ volume = {44},
+ number = {4},
+ month = jun,
+ year = {2018},
+ pages = {37:1--37:39},
+ articleno = {37},
+ numpages = {39},
+ doi = {10.1145/3182393},
+ publisher = {ACM},
 }
 """
 
