@@ -3,42 +3,71 @@ OpenMDAO Wrapper for the scipy.optimize.minimize family of local optimizers.
 """
 
 from __future__ import print_function
-from collections import OrderedDict
-import sys
 
+import sys
+from collections import OrderedDict
+from distutils.version import LooseVersion
+
+import numpy as np
+from scipy import __version__ as scipy_version
+from scipy.optimize import minimize
 from six import itervalues, iteritems, reraise
 from six.moves import range
 
-import numpy as np
-from scipy.optimize import minimize
-
 import openmdao
+import openmdao.utils.coloring as coloring_mod
 from openmdao.core.driver import Driver, RecordingDebugging
 from openmdao.utils.general_utils import warn_deprecation
-import openmdao.utils.coloring as coloring_mod
 
+# Optimizers in scipy.minimize
+_optimizers = {'Nelder-Mead', 'Powell', 'CG', 'BFGS', 'Newton-CG', 'L-BFGS-B',
+               'TNC', 'COBYLA', 'SLSQP'}
+if LooseVersion(scipy_version) >= LooseVersion("1.1"):  # Only available in newer versions
+    _optimizers.add('trust-constr')
 
-_optimizers = ['Nelder-Mead', 'Powell', 'CG', 'BFGS', 'Newton-CG', 'L-BFGS-B',
-               'TNC', 'COBYLA', 'SLSQP']
-_gradient_optimizers = ['CG', 'BFGS', 'Newton-CG', 'L-BFGS-B', 'TNC',
-                        'SLSQP', 'dogleg', 'trust-ncg']
-_bounds_optimizers = ['L-BFGS-B', 'TNC', 'SLSQP']
-_constraint_optimizers = ['COBYLA', 'SLSQP']
-_constraint_grad_optimizers = ['SLSQP']
-_eq_constraint_optimizers = ['SLSQP']
+# For 'basinhopping' and 'shgo' gradients are used only in the local minimization
+_gradient_optimizers = {'CG', 'BFGS', 'Newton-CG', 'L-BFGS-B', 'TNC', 'SLSQP', 'dogleg',
+                        'trust-ncg', 'trust-constr', 'basinhopping', 'shgo'}
+_hessian_optimizers = {'trust-constr', 'trust-ncg'}
+_bounds_optimizers = {'L-BFGS-B', 'TNC', 'SLSQP', 'trust-constr', 'dual_annealing', 'shgo',
+                      'differential_evolution', 'basinhopping'}
+_constraint_optimizers = {'COBYLA', 'SLSQP', 'trust-constr', 'shgo'}
+_constraint_grad_optimizers = _gradient_optimizers & _constraint_optimizers
+_eq_constraint_optimizers = {'SLSQP', 'trust-constr'}
+_global_optimizers = {'differential_evolution', 'basinhopping'}
+if LooseVersion(scipy_version) >= LooseVersion("1.2"):  # Only available in newer versions
+    _global_optimizers |= {'shgo', 'dual_annealing'}
+
+# Global optimizers and optimizers in minimize
+_all_optimizers = _optimizers | _global_optimizers
 
 # These require Hessian or Hessian-vector product, so they are not supported
 # right now.
-_unsupported_optimizers = ['dogleg', 'trust-ncg']
+# dual-annealing and basinhopping not supported yet
+_unsupported_optimizers = {'dogleg', 'trust-ncg'}
 
+# With "old-style" a constraint is a dictionary, with "new-style" an object
+# With "old-style" a bound is a tuple, with "new-style" a Bounds instance
+# In principle now everything can work with "old-style"
+# These settings have no effect to the optimizers implemented before SciPy 1.1
+_supports_new_style = {'trust-constr'}
+_use_new_style = True  # Recommended to set to True
 
 CITATIONS = """
-@phdthesis{hwang_thesis_2015,
-  author       = {John T. Hwang},
-  title        = {A Modular Approach to Large-Scale Design Optimization of Aerospace Systems},
-  school       = {University of Michigan},
-  year         = 2015
-}
+@article{Hwang_maud_2018
+ author = {Hwang, John T. and Martins, Joaquim R.R.A.},
+ title = "{A Computational Architecture for Coupling Heterogeneous
+          Numerical Models and Computing Coupled Derivatives}",
+ journal = "{ACM Trans. Math. Softw.}",
+ volume = {44},
+ number = {4},
+ month = jun,
+ year = {2018},
+ pages = {37:1--37:39},
+ articleno = {37},
+ numpages = {39},
+ doi = {10.1145/3182393},
+ publisher = {ACM},
 """
 
 
@@ -76,6 +105,10 @@ class ScipyOptimizeDriver(Driver):
     _obj_and_nlcons : list
         List of objective + nonlinear constraints. Used to compute total derivatives
         for all except linear constraints.
+    _dvlist : list
+        Copy of _designvars.
+    _lincongrad_cache : np.ndarray
+        Pre-calculated gradients of linear constraints.
     """
 
     def __init__(self, **kwargs):
@@ -105,11 +138,12 @@ class ScipyOptimizeDriver(Driver):
         self.opt_settings = OrderedDict()
 
         self.result = None
-        self.fail = 0
         self._grad_cache = None
         self._con_cache = None
         self._con_idx = {}
         self._obj_and_nlcons = None
+        self._dvlist = None
+        self._lincongrad_cache = None
         self.fail = False
         self.iter_count = 0
         self._exc_info = None
@@ -120,7 +154,7 @@ class ScipyOptimizeDriver(Driver):
         """
         Declare options before kwargs are processed in the init method.
         """
-        self.options.declare('optimizer', 'SLSQP', values=_optimizers,
+        self.options.declare('optimizer', 'SLSQP', values=_all_optimizers,
                              desc='Name of optimizer to use')
         self.options.declare('tol', 1.0e-6, lower=0.0,
                              desc='Tolerance for termination. For detailed '
@@ -187,6 +221,7 @@ class ScipyOptimizeDriver(Driver):
                     d['adder'] = None
                     d['scaler'] = None
                     d['size'] = meta['size']
+                    d['linear'] = True
                     self._cons[name] = d
 
     def run(self):
@@ -254,6 +289,22 @@ class ScipyOptimizeDriver(Driver):
 
                     bounds.append((p_low, p_high))
 
+        if use_bounds and (opt in _supports_new_style) and _use_new_style:
+            # For 'trust-constr' it is better to use the new type bounds, because it seems to work
+            # better (for the current examples in the tests) with the "keep_feasible" option
+            try:
+                from scipy.optimize import Bounds
+                from scipy.optimize._constraints import old_bound_to_new
+            except ImportError:
+                msg = ('The "trust-constr" optimizer is supported for SciPy 1.1.0 and above. '
+                       'The installed version is {}')
+                raise ImportError(msg.format(scipy_version))
+
+            # Convert "old-style" bounds to "new_style" bounds
+            lower, upper = old_bound_to_new(bounds)  # tuple, tuple
+            keep_feasible = self.opt_settings.get('keep_feasible_bounds', True)
+            bounds = Bounds(lb=lower, ub=upper, keep_feasible=keep_feasible)
+
         # Constraints
         constraints = []
         i = 1  # start at 1 since row 0 is the objective.  Constraints start at row 1.
@@ -266,6 +317,7 @@ class ScipyOptimizeDriver(Driver):
                 size = meta['size']
                 upper = meta['upper']
                 lower = meta['lower']
+                equals = meta['equals']
                 if 'linear' in meta and meta['linear']:
                     lincons.append(name)
                     self._con_idx[name] = lin_i
@@ -275,36 +327,65 @@ class ScipyOptimizeDriver(Driver):
                     self._con_idx[name] = i
                     i += size
 
-                # Loop over every index separately, because scipy calls each constraint by index.
-                for j in range(0, size):
-                    con_dict = {}
-                    if meta['equals'] is not None:
-                        con_dict['type'] = 'eq'
+                # In scipy constraint optimizers take constraints in two separate formats
+
+                # Type of constraints is list of NonlinearConstraint
+                if opt in _supports_new_style and _use_new_style:
+                    try:
+                        from scipy.optimize import NonlinearConstraint
+                    except ImportError:
+                        msg = ('The "trust-constr" optimizer is supported for SciPy 1.1.0 and'
+                               'above. The installed version is {}')
+                        raise ImportError(msg.format(scipy_version))
+
+                    if equals is not None:
+                        lb = ub = equals
                     else:
-                        con_dict['type'] = 'ineq'
-                    con_dict['fun'] = self._confunc
-                    if opt in _constraint_grad_optimizers:
-                        con_dict['jac'] = self._congradfunc
-                    con_dict['args'] = [name, False, j]
-                    constraints.append(con_dict)
-
-                    if isinstance(upper, np.ndarray):
-                        upper = upper[j]
-
-                    if isinstance(lower, np.ndarray):
-                        lower = lower[j]
-
-                    dblcon = (upper < openmdao.INF_BOUND) and (lower > -openmdao.INF_BOUND)
-
-                    # Add extra constraint if double-sided
-                    if dblcon:
-                        dcon_dict = {}
-                        dcon_dict['type'] = 'ineq'
-                        dcon_dict['fun'] = self._confunc
+                        lb = lower
+                        ub = upper
+                    # Loop over every index separately,
+                    # because scipy calls each constraint by index.
+                    for j in range(size):
+                        # Double-sided constraints are accepted by the algorithm
+                        args = [name, False, j]
+                        # TODO linear constraint if meta['linear']
+                        # TODO add option for Hessian
+                        con = NonlinearConstraint(fun=signature_extender(self._con_val_func, args),
+                                                  lb=lb, ub=ub,
+                                                  jac=signature_extender(self._congradfunc, args))
+                        constraints.append(con)
+                else:  # Type of constraints is list of dict
+                    # Loop over every index separately,
+                    # because scipy calls each constraint by index.
+                    for j in range(size):
+                        con_dict = {}
+                        if meta['equals'] is not None:
+                            con_dict['type'] = 'eq'
+                        else:
+                            con_dict['type'] = 'ineq'
+                        con_dict['fun'] = self._confunc
                         if opt in _constraint_grad_optimizers:
-                            dcon_dict['jac'] = self._congradfunc
-                        dcon_dict['args'] = [name, True, j]
-                        constraints.append(dcon_dict)
+                            con_dict['jac'] = self._congradfunc
+                        con_dict['args'] = [name, False, j]
+                        constraints.append(con_dict)
+
+                        if isinstance(upper, np.ndarray):
+                            upper = upper[j]
+
+                        if isinstance(lower, np.ndarray):
+                            lower = lower[j]
+
+                        dblcon = (upper < openmdao.INF_BOUND) and (lower > -openmdao.INF_BOUND)
+
+                        # Add extra constraint if double-sided
+                        if dblcon:
+                            dcon_dict = {}
+                            dcon_dict['type'] = 'ineq'
+                            dcon_dict['fun'] = self._confunc
+                            if opt in _constraint_grad_optimizers:
+                                dcon_dict['jac'] = self._congradfunc
+                            dcon_dict['args'] = [name, True, j]
+                            constraints.append(dcon_dict)
 
             # precalculate gradients of linear constraints
             if lincons:
@@ -319,23 +400,102 @@ class ScipyOptimizeDriver(Driver):
         else:
             jac = None
 
+        # Hessian calculation method for optimizers, which require it
+        if opt in _hessian_optimizers:
+            if 'hess' in self.opt_settings:
+                hess = self.opt_settings.pop('hess')
+            else:
+                # Defaults to BFGS, if not in opt_settings
+                from scipy.optimize import BFGS
+                hess = BFGS()
+        else:
+            hess = None
+
         # compute dynamic simul deriv coloring if option is set
         if coloring_mod._use_sparsity and self.options['dynamic_simul_derivs']:
             coloring_mod.dynamic_simul_coloring(self, run_model=False, do_sparsity=False)
 
         # optimize
         try:
-            result = minimize(self._objfunc, x_init,
-                              # args=(),
-                              method=opt,
-                              jac=jac,
-                              # hess=None,
-                              # hessp=None,
+            if opt in _optimizers:
+                result = minimize(self._objfunc, x_init,
+                                  # args=(),
+                                  method=opt,
+                                  jac=jac,
+                                  hess=hess,
+                                  # hessp=None,
+                                  bounds=bounds,
+                                  constraints=constraints,
+                                  tol=self.options['tol'],
+                                  # callback=None,
+                                  options=self.opt_settings)
+            elif opt == 'basinhopping':
+                from scipy.optimize import basinhopping
+
+                def fun(x):
+                    return self._objfunc(x), jac(x)
+
+                if 'minimizer_kwargs' not in self.opt_settings:
+                    self.opt_settings['minimizer_kwargs'] = {"method": "L-BFGS-B", "jac": True}
+                self.opt_settings.pop('maxiter')  # It does not have this argument
+
+                def accept_test(f_new, x_new, f_old, x_old):
+                    # Used to implement bounds besides the original functionality
+                    if bounds is not None:
+                        bound_check = all([b[0] <= xi <= b[1] for xi, b in zip(x_new, bounds)])
+                        user_test = self.opt_settings.pop('accept_test', None)  # callable
+                        # has to satisfy both the bounds and the acceptance test defined by the
+                        # user
+                        if user_test is not None:
+                            test_res = user_test(f_new, x_new, f_old, x_old)
+                            if test_res == 'force accept':
+                                return test_res
+                            else:  # result is boolean
+                                return bound_check and test_res
+                        else:  # no user acceptance test, check only the bounds
+                            return bound_check
+                    else:
+                        return True
+
+                result = basinhopping(fun, x_init,
+                                      accept_test=accept_test,
+                                      **self.opt_settings)
+            elif opt == 'dual_annealing':
+                from scipy.optimize import dual_annealing
+                self.opt_settings.pop('disp')  # It does not have this argument
+                # There is no "options" param, so "opt_settings" can be used to set the (many)
+                # keyword arguments
+                result = dual_annealing(self._objfunc,
+                                        bounds=bounds,
+                                        **self.opt_settings)
+            elif opt == 'differential_evolution':
+                from scipy.optimize import differential_evolution
+                # There is no "options" param, so "opt_settings" can be used to set the (many)
+                # keyword arguments
+                result = differential_evolution(self._objfunc,
+                                                bounds=bounds,
+                                                **self.opt_settings)
+            elif opt == 'shgo':
+                from scipy.optimize import shgo
+                kwargs = dict()
+                for param in ('minimizer_kwargs', 'sampling_method ', 'n', 'iters'):
+                    if param in self.opt_settings:
+                        kwargs[param] = self.opt_settings[param]
+                # Set the Jacobian and the Hessian to the value calculated in OpenMDAO
+                if 'minimizer_kwargs' not in kwargs or kwargs['minimizer_kwargs'] is None:
+                    kwargs['minimizer_kwargs'] = {}
+                kwargs['minimizer_kwargs'].setdefault('jac', jac)
+                kwargs['minimizer_kwargs'].setdefault('hess', hess)
+                # Objective function tolerance
+                self.opt_settings['f_tol'] = self.options['tol']
+                result = shgo(self._objfunc,
                               bounds=bounds,
                               constraints=constraints,
-                              tol=self.options['tol'],
-                              # callback=None,
-                              options=self.opt_settings)
+                              options=self.opt_settings,
+                              **kwargs)
+            else:
+                msg = 'Optimizer "{}" is not implemented yet. Choose from: {}'
+                raise NotImplementedError(msg.format(opt, _all_optimizers))
 
         # If an exception was swallowed in one of our callbacks, we want to raise it
         # rather than the cryptic message from scipy.
@@ -349,17 +509,23 @@ class ScipyOptimizeDriver(Driver):
             self._reraise()
 
         self.result = result
-        self.fail = False if self.result.success else True
 
-        if self.fail:
-            print('Optimization FAILED.')
+        if hasattr(result, 'success'):
+            self.fail = False if result.success else True
+            if self.fail:
+                print('Optimization FAILED.')
+                print(result.message)
+                print('-' * 35)
+
+            elif self.options['disp']:
+                print('Optimization Complete')
+                print('-' * 35)
+        else:
+            self.fail = True  # It is not known, so the worst option is assumed
+            print('Optimization Complete (success not known)')
             print(result.message)
             print('-' * 35)
 
-        elif self.options['disp']:
-            print('Optimization Complete')
-            print('-' * 35)
-
         return self.fail
 
     def _objfunc(self, x_new):
@@ -394,7 +560,7 @@ class ScipyOptimizeDriver(Driver):
                 model._solve_nonlinear()
 
             # Get the objective function evaluations
-            for name, obj in iteritems(self.get_objective_values()):
+            for obj in itervalues(self.get_objective_values()):
                 f_new = obj
                 break
 
@@ -410,6 +576,31 @@ class ScipyOptimizeDriver(Driver):
 
         return f_new
 
+    def _con_val_func(self, x_new, name, dbl, idx):
+        """
+        Return the value of the constraint function requested in args.
+
+        The lower or upper bound is **not** subtracted from the value. Used for optimizers,
+        which take the bounds of the constraints (e.g. trust-constr)
+
+        Parameters
+        ----------
+        x_new : ndarray
+            Array containing parameter values at new design point.
+        name : string
+            Name of the constraint to be evaluated.
+        dbl : bool
+            True if double sided constraint.
+        idx : float
+            Contains index into the constraint array.
+
+        Returns
+        -------
+        float
+            Value of the constraint function.
+        """
+        return self._con_cache[name][idx]
+
     def _confunc(self, x_new, name, dbl, idx):
         """
         Return the value of the constraint function requested in args.
@@ -444,7 +635,7 @@ class ScipyOptimizeDriver(Driver):
         if equals is not None:
             if isinstance(equals, np.ndarray):
                 equals = equals[idx]
-            return (cons[name][idx] - equals)
+            return cons[name][idx] - equals
 
         # Note, scipy defines constraints to be satisfied when positive,
         # which is the opposite of OpenMDAO.
@@ -554,6 +745,35 @@ class ScipyOptimizeDriver(Driver):
         reraise(*exc)
 
 
+def signature_extender(fcn, extra_args):
+    """
+    Closure function, which appends extra arguments to the original function call.
+
+    The first argument is the design vector. The possible extra arguments from the callback
+    of :func:`scipy.optimize.minimize` are not passed to the function.
+
+    Some algorithms take a sequence of :class:`~scipy.optimize.NonlinearConstraint` as input
+    for the constraints. For this class it is not possible to pass additional arguments.
+    With this function the signature will be correct for both scipy and the driver.
+
+    Parameters
+    ----------
+    fcn : callable
+        Function, which takes the design vector as the first argument.
+    extra_args : tuple or list
+        Extra arguments for the function
+
+    Returns
+    -------
+    callable
+        The function with the signature expected by the driver.
+    """
+    def closure(x, *args):
+        return fcn(x, *extra_args)
+
+    return closure
+
+
 class ScipyOptimizer(ScipyOptimizeDriver):
     """
     Deprecated.  Use ScipyOptimizeDriver.
