@@ -10,7 +10,7 @@ from openmdao.utils.general_utils import set_pyoptsparse_opt
 from openmdao.utils.mpi import MPI
 
 from openmdao.api import ExecComp, ExplicitComponent, Problem, \
-    Group, ParallelGroup, IndepVarComp, SqliteRecorder
+    Group, ParallelGroup, IndepVarComp, SqliteRecorder, ScipyOptimizeDriver
 from openmdao.utils.array_utils import evenly_distrib_idxs
 from openmdao.recorders.tests.sqlite_recorder_test_utils import assertDriverIterDataRecorded
 from openmdao.recorders.tests.recorder_test_utils import run_driver
@@ -20,12 +20,6 @@ if MPI:
 else:
     PETScVector = None
 
-# check that pyoptsparse is installed. if it is, try to use SLSQP.
-OPT, OPTIMIZER = set_pyoptsparse_opt('SLSQP')
-
-if OPTIMIZER:
-    from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver
-
 
 class DistributedAdder(ExplicitComponent):
     """
@@ -184,61 +178,61 @@ class DistributedRecorderTest(unittest.TestCase):
             expected_data = ((coordinate, (t0, t1), expected_outputs, None),)
             assertDriverIterDataRecorded(self, expected_data, self.eps)
 
-    @unittest.skipIf(OPT is None, "pyoptsparse is not installed")
-    @unittest.skipIf(OPTIMIZER is None, "pyoptsparse is not providing SNOPT or SLSQP")
     def test_recording_remote_voi(self):
-        prob = Problem()
+        # Create a parallel model
+        model = Group()
 
-        prob.model.add_subsystem('par', ParallelGroup())
+        model.add_subsystem('par', ParallelGroup())
+        model.par.add_subsystem('G1', Mygroup())
+        model.par.add_subsystem('G2', Mygroup())
+        model.connect('par.G1.y', 'Obj.y1')
+        model.connect('par.G2.y', 'Obj.y2')
 
-        prob.model.par.add_subsystem('G1', Mygroup())
-        prob.model.par.add_subsystem('G2', Mygroup())
+        model.add_subsystem('Obj', ExecComp('obj=y1+y2'))
+        model.add_objective('Obj.obj')
 
-        prob.model.add_subsystem('Obj', ExecComp('obj=y1+y2'))
+        # Configure driver to record VOIs on both procs
+        driver = ScipyOptimizeDriver(disp=False)
 
-        prob.model.connect('par.G1.y', 'Obj.y1')
-        prob.model.connect('par.G2.y', 'Obj.y2')
+        driver.recording_options['record_desvars'] = True
+        driver.recording_options['record_responses'] = True
+        driver.recording_options['record_objectives'] = True
+        driver.recording_options['record_constraints'] = True
+        driver.recording_options['includes'] = ['par.G1.y', 'par.G2.y']
 
-        prob.model.add_objective('Obj.obj')
-
-        prob.driver = pyOptSparseDriver()
-        prob.driver.options['optimizer'] = 'SLSQP'
-
-        prob.driver.recording_options['record_desvars'] = True
-        prob.driver.recording_options['record_responses'] = True
-        prob.driver.recording_options['record_objectives'] = True
-        prob.driver.recording_options['record_constraints'] = True
-        prob.driver.recording_options['includes'] = ['par.G1.Cy.y', 'par.G2.Cy.y']
-
-        prob.driver.add_recorder(self.recorder)
+        driver.add_recorder(self.recorder)
 
+        # Create problem and run driver
+        prob = Problem(model, driver)
         prob.setup()
         t0, t1 = run_driver(prob)
         prob.cleanup()
 
         # Since the test will compare the last case recorded, just check the
-        #   current values in the problem. This next section is about getting those values
+        # current values in the problem. This next section is about getting those values
 
         # These involve collective gathers so all ranks need to run this
         expected_outputs = prob.driver.get_design_var_values()
         expected_outputs.update(prob.driver.get_objective_values())
         expected_outputs.update(prob.driver.get_constraint_values())
 
-        # Determine the expected values for the sysincludes
-        # this gets all of the outputs but just locally
-        rrank = prob.comm.rank  # root ( aka model ) rank.
-        rowned = prob.model._owning_rank
-        # names of sysincl vars on this rank
-        local_inclnames = [n for n in prob.driver.recording_options['includes']
-                           if rrank == rowned[n]]
-        # Get values for vars on this rank
-        inputs, outputs, residuals = prob.model.get_nonlinear_vectors()
-        #   Potential local sysvars are in this
-        sysvars = outputs._views
-        # Just get the values for the sysincl vars on this rank
-        local_vars = {c: sysvars[c] for c in local_inclnames}
-        # Gather up the values for all the sysincl vars on all ranks
-        all_vars = prob.model.comm.gather(local_vars, root=0)
+        # includes for outputs are specified as promoted names but we need absolute names
+        prom2abs = model._var_allprocs_prom2abs_list['output']
+        abs_includes = [prom2abs[n][0] for n in prob.driver.recording_options['includes']]
+
+        # Absolute path names of includes on this rank
+        rrank = model.comm.rank
+        rowned = model._owning_rank
+        local_includes = [n for n in abs_includes if rrank == rowned[n]]
+
+        # Get values for all vars on this rank
+        inputs, outputs, residuals = model.get_nonlinear_vectors()
+
+        # Get values for includes on this rank
+        local_vars = {n: outputs[n] for n in local_includes}
+
+        # Gather values for includes on all ranks
+        all_vars = model.comm.gather(local_vars, root=0)
 
         if prob.comm.rank == 0:
             # Only on rank 0 do we have all the values. The all_vars variable is a list of
@@ -254,7 +248,7 @@ class DistributedRecorderTest(unittest.TestCase):
 
             expected_outputs.update(expected_includes)
 
-            coordinate = [0, 'SLSQP', (2,)]
+            coordinate = [0, 'SLSQP', (driver.iter_count-1,)]
 
             expected_data = ((coordinate, (t0, t1), expected_outputs, None),)
             assertDriverIterDataRecorded(self, expected_data, self.eps)
