@@ -440,7 +440,7 @@ class TestSqliteRecorder(unittest.TestCase):
         assertMetadataRecorded(self, prom2abs, abs2prom)
         expected_problem_metadata = {
             'connections_list_length': 11,
-            'tree_length': 4,
+            'tree_length': 6,
             'tree_children_length': 7,
             'abs2prom': abs2prom,
         }
@@ -478,8 +478,9 @@ class TestSqliteRecorder(unittest.TestCase):
         # Quick check to see that keys and values were recorded
         for key in ['root', 'px', 'pz', 'd1', 'd2', 'obj_cmp', 'con_cmp1', 'con_cmp2']:
             self.assertTrue(key in cr.system_metadata.keys())
-            value = cr.system_metadata[key]['component_options']['assembled_jac_type']
-            self.assertEqual(value, 'csc')  # quick check only. Too much to check exhaustively
+
+        value = cr.system_metadata['root']['component_options']['assembled_jac_type']
+        self.assertEqual(value, 'csc')  # quick check only. Too much to check exhaustively
 
         # second check to see if not recorded recursively, when option set to False
         prob = Problem(model=SellarDerivatives())
@@ -513,8 +514,9 @@ class TestSqliteRecorder(unittest.TestCase):
         # Quick check to see that keys and values were recorded
         for key in ['root', 'px', 'pz', 'd1', 'd2', 'obj_cmp', 'con_cmp1', 'con_cmp2']:
             self.assertTrue(key in cr.system_metadata.keys())
-            value = cr.system_metadata[key]['component_options']['assembled_jac_type']
-            self.assertEqual(value, 'csc')  # quick check only. Too much to check exhaustively
+
+        value = cr.system_metadata['root']['component_options']['assembled_jac_type']
+        self.assertEqual(value, 'csc')  # quick check only. Too much to check exhaustively
 
         prob = Problem(model=SellarDerivatives())
         prob.setup()
@@ -554,6 +556,8 @@ class TestSqliteRecorder(unittest.TestCase):
         model.recording_options['record_metadata'] = True
         model.add_recorder(self.recorder)
 
+        model.nonlinear_solver.options['use_apply_nonlinear'] = True
+
         d1 = model.d1  # SellarDis1withDerivatives, an ExplicitComp
         d1.recording_options['record_inputs'] = True
         d1.recording_options['record_outputs'] = True
@@ -630,7 +634,7 @@ class TestSqliteRecorder(unittest.TestCase):
         driver.recording_options['record_objectives'] = True
         driver.recording_options['record_constraints'] = True
         driver.recording_options['includes'] = ['*']
-        driver.recording_options['excludes'] = ['p2*']
+        driver.recording_options['excludes'] = ['y*']
         driver.add_recorder(self.recorder)
 
         prob.setup()
@@ -676,7 +680,7 @@ class TestSqliteRecorder(unittest.TestCase):
         driver.recording_options['record_objectives'] = True
         driver.recording_options['record_constraints'] = True
         driver.recording_options['includes'] = ['*']
-        driver.recording_options['excludes'] = ['p2*']
+        driver.recording_options['excludes'] = ['y*']
 
         prob.set_solver_print(0)
         t0, t1 = run_driver(prob)
@@ -713,6 +717,8 @@ class TestSqliteRecorder(unittest.TestCase):
         model.recording_options['record_metadata'] = True
         model.add_recorder(self.recorder)
 
+        model.mda.nonlinear_solver.options['use_apply_nonlinear'] = True
+
         pz = model.pz  # IndepVarComp which is an ExplicitComponent
         pz.recording_options['record_inputs'] = True
         pz.recording_options['record_outputs'] = True
@@ -778,6 +784,7 @@ class TestSqliteRecorder(unittest.TestCase):
         prob.setup()
 
         nl = prob.model.nonlinear_solver
+        nl.options['use_apply_nonlinear'] = True
         nl.recording_options['record_abs_error'] = True
         nl.recording_options['record_rel_error'] = True
         nl.recording_options['record_solver_residuals'] = True
@@ -862,12 +869,12 @@ class TestSqliteRecorder(unittest.TestCase):
             0,
             'Driver', (0,),
             'root._solve_nonlinear', (0,),
-            'NewtonSolver', (3,),
+            'NewtonSolver', (2,),
             'ArmijoGoldsteinLS', (2,)
         ]
 
-        expected_abs_error = 5.46371836663e-11
-        expected_rel_error = 0.120259301544
+        expected_abs_error = 5.6736837450444e-12
+        expected_rel_error = 0.0047475363051265665
 
         expected_solver_output = {
             "con_cmp1.con1": [-22.42830237],
@@ -961,6 +968,7 @@ class TestSqliteRecorder(unittest.TestCase):
 
         prob.model.nonlinear_solver.add_recorder(self.recorder)
         prob.model.nonlinear_solver.recording_options['record_solver_residuals'] = True
+        prob.model.nonlinear_solver.options['use_apply_nonlinear'] = True
 
         prob.set_solver_print(0)
         t0, t1 = run_driver(prob)
@@ -1354,6 +1362,8 @@ class TestSqliteRecorder(unittest.TestCase):
         # Need to do recursive adding of recorders AFTER setup
         prob.model.add_recorder(self.recorder, recurse=True)
 
+        prob.model.mda.nonlinear_solver.options['use_apply_nonlinear'] = True
+
         prob.run_model()
         prob.cleanup()
 
@@ -1387,6 +1397,8 @@ class TestSqliteRecorder(unittest.TestCase):
         prob = SellarProblem(SellarDerivativesGrouped, nonlinear_solver=NonlinearRunOnce)
         prob.setup(mode='rev')
 
+        prob.model.mda.nonlinear_solver.options['use_apply_nonlinear'] = True
+
         prob.model.add_recorder(self.recorder, recurse=True)
 
         prob.run_model('Run1')
@@ -1438,7 +1450,7 @@ class TestSqliteRecorder(unittest.TestCase):
         driver.recording_options['record_objectives'] = True
         driver.recording_options['record_constraints'] = True
         driver.recording_options['record_inputs'] = False
-        driver.recording_options['includes'] = ['mda.d2.y2']
+        driver.recording_options['includes'] = ['y2']
         driver.add_recorder(self.recorder)
 
         prob.setup()
@@ -1745,7 +1757,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
 
         # access the model tree stored in metadata
         self.assertEqual(set(cr.problem_metadata['tree'].keys()),
-                         {'name', 'type', 'subsystem_type', 'children'})
+                         {'name', 'type', 'subsystem_type', 'children', 'linear_solver', 'nonlinear_solver'})
         self.assertEqual(cr.problem_metadata['tree']['name'], 'root')
         self.assertEqual(sorted([child["name"] for child in cr.problem_metadata['tree']["children"]]),
                          ['con_cmp1', 'con_cmp2', 'd1', 'd2', 'obj_cmp', 'px', 'pz'])
@@ -1820,19 +1832,16 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
 
         # options for system 'd1', with second option excluded
         self.assertEqual(str(metadata['d1']['component_options']),
-            "================== ======= ================= ================ ======================================\n"
-            "Option             Default Acceptable Values Acceptable Types Description                           \n"
-            "================== ======= ================= ================ ======================================\n"
-            "assembled_jac_type csc     ['csc', 'dense']  N/A              Linear solver(s) in this group, if usi\n"
-            "                                                              ng an assembled jacobian, will use thi\n"
-            "                                                              s type.\n"
-            "distributed        False   N/A               N/A              True if the component has variables th\n"
-            "                                                              at are distributed across multiple pro\n"
-            "                                                              cesses.\n"
-            "options value 1    1       N/A               N/A                                                    \n"
-            "================== ======= ================= ================ ======================================")
+            "=============== ======= ================= ================ =========================================\n"
+            "Option          Default Acceptable Values Acceptable Types Description                              \n"
+            "=============== ======= ================= ================ =========================================\n"
+            "distributed     False   N/A               N/A              True if the component has variables that \n"
+            "                                                           are distributed across multiple processes\n"
+            "                                                           .\n"
+            "options value 1 1       N/A               N/A                                                       \n"
+            "=============== ======= ================= ================ =========================================")
 
-        self.assertEqual(metadata['d1']['component_options']['assembled_jac_type'], 'csc')
+        self.assertEqual(metadata['d1']['component_options']['distributed'], False)
 
     def test_feature_system_options(self):
         from openmdao.api import Problem, SqliteRecorder, CaseReader
@@ -1848,6 +1857,8 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         obj_cmp.recording_options['includes'] = ['*']
         obj_cmp.recording_options['excludes'] = ['obj_cmp.x']
 
+        prob.model.nonlinear_solver.options['use_apply_nonlinear'] = True
+
         prob.run_model()
         prob.cleanup()
 
@@ -1865,6 +1876,8 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         from openmdao.api import Problem, ScipyOptimizeDriver, SqliteRecorder, CaseReader
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
+        import numpy as np
+
         prob = Problem(model=SellarDerivatives())
 
         model = prob.model
@@ -1916,6 +1929,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         solver = prob.model.nonlinear_solver
         solver.add_recorder(recorder)
         solver.recording_options['record_abs_error'] = True
+        solver.options['use_apply_nonlinear'] = True
 
         prob.run_model()
         prob.cleanup()
@@ -1998,6 +2012,8 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         from openmdao.api import Problem, ScipyOptimizeDriver, SqliteRecorder
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
+        import numpy as np
+
         prob = Problem(model=SellarDerivatives())
 
         model = prob.model
@@ -2109,6 +2125,8 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         from openmdao.api import Problem, SqliteRecorder, ScipyOptimizeDriver, CaseReader
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
+        import numpy as np
+
         prob = Problem(model=SellarDerivatives())
 
         model = prob.model
@@ -2177,6 +2195,8 @@ class TestFeatureBasicRecording(unittest.TestCase):
         from openmdao.api import Problem, ScipyOptimizeDriver, SqliteRecorder
         from openmdao.test_suite.components.sellar_feature import SellarMDA
 
+        import numpy as np
+
         # create our Sellar problem
         prob = Problem(model=SellarMDA())
 
