@@ -4,7 +4,6 @@ Define the BroydenSolver class.
 Based on implementation in Scipy via OpenMDAO 0.8x with improvements based on NPSS solver.
 """
 from __future__ import print_function
-from copy import deepcopy
 import warnings
 from six.moves import range
 
@@ -283,8 +282,8 @@ class BroydenSolver(NonlinearSolver):
         """
         system = self._system
         if self.options['debug_print']:
-            self._err_cache['inputs'] = deepcopy(self._system._inputs)
-            self._err_cache['outputs'] = deepcopy(self._system._outputs)
+            self._err_cache['inputs'] = self._system._inputs._copy_views()
+            self._err_cache['outputs'] = self._system._outputs._copy_views()
 
         # Convert local storage if we are under complex step.
         if system.under_complex_step:
@@ -313,14 +312,10 @@ class BroydenSolver(NonlinearSolver):
         self.xm = self.get_states()
 
         with Recording('Broyden', 0, self):
-
             self._solver_info.append_solver()
 
             # should call the subsystems solve before computing the first residual
-            for isub, subsys in enumerate(system._subsystems_myproc):
-                system._transfer('nonlinear', 'fwd', isub)
-                subsys._solve_nonlinear()
-                system._check_reconf_update()
+            self._gs_iter()
 
             self._solver_info.pop()
 
@@ -350,7 +345,7 @@ class BroydenSolver(NonlinearSolver):
         else:
             return np.sum(fxm**2) ** 0.5
 
-    def _iter_execute(self):
+    def _single_iteration(self):
         """
         Perform the operations in the iteration loop.
         """
@@ -377,13 +372,7 @@ class BroydenSolver(NonlinearSolver):
         # Run the model.
         with Recording('Broyden', 0, self):
             self._solver_info.append_solver()
-
-            for isub, subsys in enumerate(system._subsystems_allprocs):
-                system._transfer('nonlinear', 'fwd', isub)
-
-                if subsys in system._subsystems_myproc:
-                    subsys._solve_nonlinear()
-
+            self._gs_iter()
             self._solver_info.pop()
 
         self._run_apply()
