@@ -1,23 +1,22 @@
 """Test the Newton nonlinear solver. """
 
 import unittest
-import warnings
+
 import numpy as np
 
 from openmdao.api import Group, Problem, IndepVarComp, LinearBlockGS, \
     NewtonSolver, ExecComp, ScipyKrylov, ImplicitComponent, \
     DirectSolver, AnalysisError
-from openmdao.utils.assert_utils import assert_rel_error
+from openmdao.core.tests.test_discrete import InternalDiscreteGroup
+from openmdao.solvers.linesearch.backtracking import ArmijoGoldsteinLS
 from openmdao.test_suite.components.double_sellar import DoubleSellar, DoubleSellarImplicit, \
      SubSellar
+from openmdao.test_suite.components.implicit_newton_linesearch import ImplCompTwoStates
 from openmdao.test_suite.components.sellar import SellarDerivativesGrouped, \
      SellarNoDerivatives, SellarDerivatives, SellarStateConnection, StateConnection, \
      SellarDis1withDerivatives, SellarDis2withDerivatives
+from openmdao.utils.assert_utils import assert_rel_error, assert_warning
 
-from openmdao.solvers.linesearch.backtracking import ArmijoGoldsteinLS
-from openmdao.test_suite.components.implicit_newton_linesearch \
-    import ImplCompTwoStates, ImplCompTwoStatesArrays
-from openmdao.core.tests.test_discrete import InternalDiscreteGroup
 
 class TestNewton(unittest.TestCase):
 
@@ -91,16 +90,14 @@ class TestNewton(unittest.TestCase):
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = ScipyKrylov()
 
-        msg = "The 'line_search' attribute provides backwards compatibility with OpenMDAO 1.x ; use 'linesearch' instead."
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            top.model.nonlinear_solver.line_search = ArmijoGoldsteinLS(bound_enforcement='vector')
-            self.assertEqual(str(w[-1].message), msg)
+        msg = "The 'line_search' attribute provides backwards compatibility with OpenMDAO 1.x ; " \
+              "use 'linesearch' instead."
 
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
+        with assert_warning(DeprecationWarning, msg):
+            top.model.nonlinear_solver.line_search = ArmijoGoldsteinLS(bound_enforcement='vector')
+
+        with assert_warning(DeprecationWarning, msg):
             ls = top.model.nonlinear_solver.line_search
-            self.assertEqual(str(w[-1].message), msg)
 
         ls.options['maxiter'] = 10
         ls.options['alpha'] = 1.0
@@ -581,8 +578,8 @@ class TestNewton(unittest.TestCase):
                 super(CountNewton, self).__init__(**kwargs)
                 self.total_count = 0
 
-            def _iter_execute(self):
-                super(CountNewton, self)._iter_execute()
+            def _single_iteration(self):
+                super(CountNewton, self)._single_iteration()
                 self.total_count += 1
 
         class CountDS(DirectSolver):
