@@ -11,8 +11,6 @@ import sys
 
 import numpy as np
 
-from copy import deepcopy
-
 from openmdao.core.analysis_error import AnalysisError
 from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.recorders.recording_manager import RecordingManager
@@ -343,18 +341,9 @@ class Solver(object):
         """
         pass
 
-    def _run_iterator(self):
+    def _solve(self):
         """
         Run the iterative solver.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
         maxiter = self.options['maxiter']
         atol = self.options['atol']
@@ -370,10 +359,9 @@ class Solver(object):
 
         self._mpi_print(self._iter_count, norm, norm / norm0)
 
-        while self._iter_count < maxiter and \
-                norm > atol and norm / norm0 > rtol:
+        while self._iter_count < maxiter and norm > atol and norm / norm0 > rtol:
             with Recording(type(self).__name__, self._iter_count, self) as rec:
-                self._iter_execute()
+                self._single_iteration()
                 self._iter_count += 1
                 self._run_apply()
                 norm = self._iter_get_norm()
@@ -387,12 +375,9 @@ class Solver(object):
                 norm0 = 1
             self._mpi_print(self._iter_count, norm, norm / norm0)
 
-        fail = (np.isinf(norm) or np.isnan(norm) or
-                (norm > atol and norm / norm0 > rtol))
-
         if self._system.comm.rank == 0 or os.environ.get('USE_PROC_FILES'):
             prefix = self._solver_info.prefix + self.SOLVER
-            if fail:
+            if np.isinf(norm) or np.isnan(norm) or (norm > atol and norm / norm0 > rtol):
                 if iprint > -1:
                     msg = ' Failed to Converge in {} iterations'.format(self._iter_count)
                     print(prefix + msg)
@@ -407,8 +392,6 @@ class Solver(object):
             elif iprint == 2:
                 print(prefix + ' Converged')
 
-        return fail, norm, norm / norm0
-
     def _iter_initialize(self):
         """
         Perform any necessary pre-processing operations.
@@ -422,29 +405,12 @@ class Solver(object):
         """
         pass
 
-    def _iter_execute(self):
-        """
-        Perform the operations in the iteration loop.
-        """
-        pass
-
     def _run_apply(self):
         """
         Run the appropriate apply method on the system.
         """
         pass
 
-    def _iter_get_norm(self):
-        """
-        Return the norm of the residual.
-
-        Returns
-        -------
-        float
-            norm.
-        """
-        pass
-
     def _linearize(self):
         """
         Perform any required linearization operations such as matrix factorization.
@@ -462,21 +428,6 @@ class Solver(object):
         """
         return True
 
-    def solve(self):
-        """
-        Run the solver.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
-        """
-        pass
-
     def __str__(self):
         """
         Return a string representation of the solver.
@@ -504,17 +455,13 @@ class Solver(object):
 
         # Get the data
         data = {}
-        # if self.options['record_abs_error'] or self.options['record_rel_error']:
-        #     norm = self._iter_get_norm()
 
         if self.recording_options['record_abs_error']:
-            # data['abs'] = norm
             data['abs'] = kwargs.get('abs')
         else:
             data['abs'] = None
 
         if self.recording_options['record_rel_error']:
-            # data['rel'] = norm / self._norm0
             data['rel'] = kwargs.get('rel')
         else:
             data['rel'] = None
@@ -606,49 +553,15 @@ class NonlinearSolver(Solver):
     def solve(self):
         """
         Run the solver.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
-        raised = False
         try:
-            fail, abs_err, rel_err = self._run_iterator()
+            self._solve()
         except Exception:
-            fail = True
-            raised = True
             exc = sys.exc_info()
-
-        if fail and self.options['debug_print']:
-            coord = self._recording_iter.get_formatted_iteration_coordinate()
-
-            out_str = "\n# Inputs and outputs at start of iteration '%s':\n" % coord
-            for vec_type, vec in iteritems(self._err_cache):
-                out_str += '\n'
-                out_str += '# %s %ss\n' % (vec._name, vec._typ)
-                out_str += pprint.pformat(vec._views)
-                out_str += '\n'
-
-            print(out_str)
-
-            filename = coord.replace('._solve_nonlinear', '')
-            filename = re.sub('[^0-9a-zA-Z]', '_', filename) + '.dat'
-            with open(filename, 'w') as f:
-                f.write(out_str)
-                print("Inputs and outputs at start of iteration have been "
-                      "saved to '%s'." % filename)
-                sys.stdout.flush()
-
-        if raised:
+            if self.options['debug_print']:
+                self._print_exc_debug_info()
             reraise(*exc)
 
-        return fail, abs_err, rel_err
-
     def _iter_initialize(self):
         """
         Perform any necessary pre-processing operations.
@@ -661,8 +574,8 @@ class NonlinearSolver(Solver):
             error at the first iteration.
         """
         if self.options['debug_print']:
-            self._err_cache['inputs'] = deepcopy(self._system._inputs)
-            self._err_cache['outputs'] = deepcopy(self._system._outputs)
+            self._err_cache['inputs'] = self._system._inputs._copy_views()
+            self._err_cache['outputs'] = self._system._outputs._copy_views()
 
         if self.options['maxiter'] > 0:
             self._run_apply()
@@ -697,12 +610,46 @@ class NonlinearSolver(Solver):
         """
         Raise an exception if any discrete outputs exist in our System.
         """
-        system = self._system
-
-        if system._var_allprocs_discrete['output']:
+        if self._system._var_allprocs_discrete['output']:
             raise RuntimeError("System '%s' has a %s solver and contains discrete outputs %s." %
-                               (system.pathname, type(self).__name__,
-                                sorted(system._var_allprocs_discrete['output'])))
+                               (self._system.pathname, type(self).__name__,
+                                sorted(self._system._var_allprocs_discrete['output'])))
+
+    def _print_exc_debug_info(self):
+        coord = self._recording_iter.get_formatted_iteration_coordinate()
+
+        out_str = "\n# Inputs and outputs at start of iteration '%s':\n" % coord
+        for vec_type, views in iteritems(self._err_cache):
+            out_str += '\n'
+            out_str += '# nonlinear %s\n' % vec_type
+            out_str += pprint.pformat(views)
+            out_str += '\n'
+
+        print(out_str)
+
+        filename = coord.replace('._solve_nonlinear', '')
+        filename = re.sub('[^0-9a-zA-Z]', '_', filename) + '.dat'
+        with open(filename, 'w') as f:
+            f.write(out_str)
+            print("Inputs and outputs at start of iteration have been "
+                  "saved to '%s'." % filename)
+            sys.stdout.flush()
+
+    def _gs_iter(self):
+        """
+        Perform a Gauss-Seidel iteration over this Solver's subsystems.
+        """
+        system = self._system
+        if system._subsystems_allprocs:
+            loc = system._loc_subsys_map
+
+        for isub, subsys in enumerate(system._subsystems_allprocs):
+            system._transfer('nonlinear', 'fwd', isub)
+
+            if subsys.name in loc:
+                subsys._solve_nonlinear()
+
+            system._check_reconf_update(subsys)
 
 
 class LinearSolver(Solver):
@@ -769,18 +716,6 @@ class LinearSolver(Solver):
             depth of the current system (already incremented).
         """
         super(LinearSolver, self)._setup_solvers(system, depth)
-
-        if self._mode == 'fwd':
-            b_vecs = system._vectors['residual']
-        else:  # rev
-            b_vecs = system._vectors['output']
-
-        self._rhs_vecs = {}
-
-        if system._use_derivatives:
-            for vec_name in system._lin_rel_vec_name_list:
-                self._rhs_vecs[vec_name] = b_vecs[vec_name]._data.copy()
-
         if self.options['assemble_jac'] and not self.supports['assembled_jac']:
             raise RuntimeError("Linear solver '%s' in system '%s' doesn't support assembled "
                                "jacobians." % (self.SOLVER, system.pathname))
@@ -797,49 +732,8 @@ class LinearSolver(Solver):
             'fwd' or 'rev'.
         rel_systems : set of str
             Set of names of relevant systems based on the current linear solve.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            initial error.
-        float
-            error at the first iteration.
         """
-        self._vec_names = vec_names
-        self._rel_systems = rel_systems
-        self._mode = mode
-        return self._run_iterator()
-
-    def _iter_initialize(self):
-        """
-        Perform any necessary pre-processing operations.
-
-        Returns
-        -------
-        float
-            initial error.
-        float
-            error at the first iteration.
-        """
-        system = self._system
-
-        if self._mode == 'fwd':
-            b_vecs = system._vectors['residual']
-        else:  # rev
-            b_vecs = system._vectors['output']
-
-        for vec_name in self._vec_names:
-            self._rhs_vecs[vec_name][:] = b_vecs[vec_name]._data
-
-        if self.options['maxiter'] > 1:
-            self._run_apply()
-            norm = self._iter_get_norm()
-        else:
-            norm = 1.0
-        norm0 = norm if norm != 0.0 else 1.0
-        return norm0, norm
+        raise NotImplementedError("class %s does not implement solve()." % (type(self).__name__))
 
     def _run_apply(self):
         """
@@ -856,10 +750,106 @@ class LinearSolver(Solver):
         finally:
             self._recording_iter.stack.pop()
 
+    def _set_complex_step_mode(self, active):
+        """
+        Turn on or off complex stepping mode.
+
+        Recurses to turn on or off complex stepping mode in all subsystems and their vectors.
+
+        Parameters
+        ----------
+        active : bool
+            Complex mode flag; set to True prior to commencing complex step.
+        """
+        pass
+
+
+class BlockLinearSolver(LinearSolver):
+    """
+    A base class for LinearBlockGS and LinearBlockJac.
+    """
+
+    def _declare_options(self):
+        """
+        Declare options before kwargs are processed in the init method.
+        """
+        super(BlockLinearSolver, self)._declare_options()
+        self.supports['assembled_jac'] = False
+
+    def _setup_solvers(self, system, depth):
+        """
+        Assign system instance, set depth, and optionally perform setup.
+
+        Parameters
+        ----------
+        system : <System>
+            pointer to the owning system.
+        depth : int
+            depth of the current system (already incremented).
+        """
+        super(BlockLinearSolver, self)._setup_solvers(system, depth)
+        if system._use_derivatives:
+            self._create_rhs_vecs()
+
+    def _create_rhs_vecs(self):
+        self._rhs_vecs = rhs = {}
+        for vec_name in self._system._lin_rel_vec_name_list:
+            if self._mode == 'fwd':
+                rhs[vec_name] = self._system._vectors['residual'][vec_name]._data.copy()
+            else:
+                rhs[vec_name] = self._system._vectors['output'][vec_name]._data.copy()
+
+    def _update_rhs_vecs(self):
+        for vec_name in self._system._lin_rel_vec_name_list:
+            if self._mode == 'fwd':
+                self._rhs_vecs[vec_name][:] = self._system._vectors['residual'][vec_name]._data
+            else:
+                self._rhs_vecs[vec_name][:] = self._system._vectors['output'][vec_name]._data
+
+    def _set_complex_step_mode(self, active):
+        """
+        Turn on or off complex stepping mode.
+
+        Recurses to turn on or off complex stepping mode in all subsystems and their vectors.
+
+        Parameters
+        ----------
+        active : bool
+            Complex mode flag; set to True prior to commencing complex step.
+        """
+        for vec_name in self._system._lin_rel_vec_name_list:
+            if active:
+                self._rhs_vecs[vec_name] = self._rhs_vecs[vec_name].astype(np.complex)
+            else:
+                self._rhs_vecs[vec_name] = self._rhs_vecs[vec_name].real
+
+    def _iter_initialize(self):
+        """
+        Perform any necessary pre-processing operations.
+
+        Returns
+        -------
+        float
+            initial error.
+        float
+            error at the first iteration.
+        """
+        self._update_rhs_vecs()
+        if self.options['maxiter'] > 1:
+            self._run_apply()
+            norm = self._iter_get_norm()
+        else:
+            norm = 1.0
+        norm0 = norm if norm != 0.0 else 1.0
+        return norm0, norm
+
     def _iter_get_norm(self):
         """
         Return the norm of the residual.
 
+        Note: This has the side effect of modifying the residual vector in fwd mode
+        and the output vector in rev mode.
+
         Returns
         -------
         float
@@ -873,22 +863,26 @@ class LinearSolver(Solver):
             b_vecs = system._vectors['output']
 
         norm = 0
-        for vec_name in self._vec_names:
-            if vec_name in system._rel_vec_names:
-                b_vecs[vec_name]._data -= self._rhs_vecs[vec_name]
-                norm += b_vecs[vec_name].get_norm()**2
+        for vec_name in system._lin_rel_vec_name_list:
+            b_vecs[vec_name]._data -= self._rhs_vecs[vec_name]
+            norm += b_vecs[vec_name].get_norm()**2
 
         return norm ** 0.5
 
-
-class BlockLinearSolver(LinearSolver):
-    """
-    A base class for LinearBlockGS and LinearBlockJac.
-    """
-
-    def _declare_options(self):
+    def solve(self, vec_names, mode, rel_systems=None):
         """
-        Declare options before kwargs are processed in the init method.
+        Run the solver.
+
+        Parameters
+        ----------
+        vec_names : [str, ...]
+            list of names of the right-hand-side vectors.
+        mode : str
+            'fwd' or 'rev'.
+        rel_systems : set of str
+            Set of names of relevant systems based on the current linear solve.
         """
-        super(BlockLinearSolver, self)._declare_options()
-        self.supports['assembled_jac'] = False
+        self._vec_names = vec_names
+        self._rel_systems = rel_systems
+        self._mode = mode
+        self._solve()
